package DAOTests;

import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class ConnectionsDAOTests {

    @Autowired
    private ConnectionsDAO connectionsDAO;

    private String workflowId = "WE1"; // from test_data.sql
    private String executionId = "SE1";// from test_data.sql
    private String connectionId = "OracleTestDB"; // from test_data.sql

    private final Long ONE = 1L;

    @Test
    public void save_addingNewConnection_connectionAdded(){
        String newConnectionId = "Connection1";

        Connection connection = new Connection(
                null,
                newConnectionId,
                executionId,
                null
        );

        Long insertedRows = connectionsDAO.save(connection);

        assertEquals(ONE, insertedRows);

        Long connDbId = connection.getConnectionDbId();
        Connection foundConnection = connectionsDAO.find(connDbId);

        assertEquals(connection.getConnectionId(), foundConnection.getConnectionId());
        assertEquals(connection.getScenarioExecutionId(), foundConnection.getScenarioExecutionId());
        assertEquals(connection.getPackageName(), foundConnection.getPackageName());
    }

    @Test
    public void delete_deletingExistingConnection_connectionDeleted(){
        Connection connection = new Connection(
                null,
                connectionId,
                executionId,
                null
        );

        Long connDbId = connectionsDAO.getIdByData(connection);

        Long deletedRows = connectionsDAO.delete(connection);
        assertEquals(ONE, deletedRows);

        Connection shouldBeNullConnection = connectionsDAO.find(connDbId);
        assertNull(shouldBeNullConnection);
    }

    @Test
    public void find_findExistingConnection_connectionFound(){
        Connection connection = new Connection(
                null,
                connectionId,
                executionId,
                null
        );

        Long connDbId = connectionsDAO.getIdByData(connection);
        assertNotNull(connDbId);

        Connection foundConnection = connectionsDAO.find(connDbId);

        assertEquals(connection.getConnectionId(), foundConnection.getConnectionId());
        assertEquals(connection.getScenarioExecutionId(), foundConnection.getScenarioExecutionId());
        assertEquals(connection.getPackageName(), foundConnection.getPackageName());
    }

    @Test
    public void getAll_getAllExistingConnections_oneConnectionFound(){
        Connection connection = new Connection(
                null,
                connectionId,
                executionId,
                null
        );

        List<Connection> connections = connectionsDAO.getAll();

        assertEquals(ONE, new Long(connections.size()));

        Connection foundConnection = connections.get(0);

        assertEquals(connection.getConnectionId(), foundConnection.getConnectionId());
        assertEquals(connection.getScenarioExecutionId(), foundConnection.getScenarioExecutionId());
        assertEquals(connection.getPackageName(), foundConnection.getPackageName());
    }

    @Test(expected = DataIntegrityViolationException.class)
    public void save_insertConnectionWithoutExistingScenarioExecution_shouldThrowConstraintViolation(){
        Connection connection = new Connection(
                null,
                workflowId,
                "SE2",
                null
        );

        Long insertedRows = connectionsDAO.save(connection);
    }

}
