package DAOTests;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class ErrorDefinitionDAOTests {

    @Autowired
    private ErrorDefinitionDAO errorDefinitionDAO;

    private String errorCategory = "TEST_CATEGORY";
    private String errorType = "TEST_ERROR";
    private String userMessage = "Test user message.";
    private String technicalMessage = "Test technical message.";
    private String solution = "Test solution.";
    private String impact = "Test impact.";
    private String logLevel = "ERROR";

    private final Long ONE = 1L;

    @Test
    public void save_addNewErrorDefinition_errorDefinitionAdded(){
        String newErrorType = "TEST_ERROR2";

        ErrorDefinition errorDefinition = new ErrorDefinition(
                errorCategory,
                newErrorType,
                userMessage,
                technicalMessage,
                solution,
                impact,
                logLevel
        );

        Long insertedRows = errorDefinitionDAO.save(errorDefinition);

        assertEquals(ONE, insertedRows);

        Long id = errorDefinition.getErrorDefinitionId();
        ErrorDefinition foundErrorDefinition = errorDefinitionDAO.find(id);

        assertEquals(errorDefinition.getErrorCategory(), foundErrorDefinition.getErrorCategory());
        assertEquals(errorDefinition.getErrorType(), foundErrorDefinition.getErrorType());
        assertEquals(errorDefinition.getUserMessage(), foundErrorDefinition.getUserMessage());
        assertEquals(errorDefinition.getTechnicalMessage(), foundErrorDefinition.getTechnicalMessage());
        assertEquals(errorDefinition.getSolution(), foundErrorDefinition.getSolution());
        assertEquals(errorDefinition.getImpact(), foundErrorDefinition.getImpact());
        assertEquals(errorDefinition.getLogLevel(), foundErrorDefinition.getLogLevel());
    }

    @Test
    public void delete_deleteExistingErrorDefinition_errorDefinitionDeleted(){
        ErrorDefinition errorDefinition = new ErrorDefinition(
                errorCategory,
                errorType,
                userMessage,
                technicalMessage,
                solution,
                impact,
                logLevel
        );

        Long dbId = errorDefinitionDAO.getIdByData(errorDefinition);
        assertNotNull(dbId);

        Long deletedRows = errorDefinitionDAO.delete(Arrays.asList(dbId));
        assertEquals(ONE, deletedRows);

        ErrorDefinition foundErrorDefinition = errorDefinitionDAO.find(dbId);
        assertNull(foundErrorDefinition);
    }

    @Test
    public void find_getExistingErrorDefinition_errorDefinitionFound(){
        ErrorDefinition errorDefinition = new ErrorDefinition(
                errorCategory,
                errorType,
                userMessage,
                technicalMessage,
                solution,
                impact,
                logLevel
        );

        Long dbId = errorDefinitionDAO.getIdByData(errorDefinition);
        assertNotNull(dbId);

        ErrorDefinition foundErrorDefinition = errorDefinitionDAO.find(dbId);

        assertEquals(errorDefinition.getErrorCategory(), foundErrorDefinition.getErrorCategory());
        assertEquals(errorDefinition.getErrorType(), foundErrorDefinition.getErrorType());
        assertEquals(errorDefinition.getUserMessage(), foundErrorDefinition.getUserMessage());
        assertEquals(errorDefinition.getTechnicalMessage(), foundErrorDefinition.getTechnicalMessage());
        assertEquals(errorDefinition.getSolution(), foundErrorDefinition.getSolution());
        assertEquals(errorDefinition.getImpact(), foundErrorDefinition.getImpact());
        assertEquals(errorDefinition.getLogLevel(), foundErrorDefinition.getLogLevel());
    }

    @Test
    public void getAll_getAllExistingErrorDefinition_oneDefinitionFound(){
        ErrorDefinition errorDefinition = new ErrorDefinition(
                errorCategory,
                errorType,
                userMessage,
                technicalMessage,
                solution,
                impact,
                logLevel
        );

        List<Long> allDefinitions = errorDefinitionDAO.getAll();
        assertEquals(ONE, new Long(allDefinitions.size()));

        Long dbId = allDefinitions.get(0);

        ErrorDefinition foundErrorDefinition = errorDefinitionDAO.find(dbId);

        assertEquals(errorDefinition.getErrorCategory(), foundErrorDefinition.getErrorCategory());
        assertEquals(errorDefinition.getErrorType(), foundErrorDefinition.getErrorType());
        assertEquals(errorDefinition.getUserMessage(), foundErrorDefinition.getUserMessage());
        assertEquals(errorDefinition.getTechnicalMessage(), foundErrorDefinition.getTechnicalMessage());
        assertEquals(errorDefinition.getSolution(), foundErrorDefinition.getSolution());
        assertEquals(errorDefinition.getImpact(), foundErrorDefinition.getImpact());
        assertEquals(errorDefinition.getLogLevel(), foundErrorDefinition.getLogLevel());
    }
}
