package DAOTests;

import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageDAO;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class PackageDAOTests {

    @Autowired
    private PackageDAO packageDAO;

    private final String EMPTY_STRING = "";

    private String packageName1 = "My Package 1";
    private String application1 = "Manta CLI";

    private String packageName2 = "My Package 2";
    private String technology2 = "Oracle";
    private String phase2 = "Analysis";
    private String scenarioName2 = "Oracle DDL Dataflow Analysis";
    private String connection2 = "OracleTestDB";

    private final Long ONE = 1L;

    @Test
    public void delete_deleteExistingPackage_deleted(){
        Long deletedRows = packageDAO.delete(packageName1);
        assertEquals(ONE, deletedRows);

        List<UploadedPackage> uploadedPackageList = packageDAO.getAll();
        assertEquals(ONE, new Long(uploadedPackageList.size()));
    }

    @Test
    public void getAll_getAllPackages_shouldBeTwo(){
        List<UploadedPackage> uploadedPackageList = packageDAO.getAll();
        assertEquals(2, uploadedPackageList.size());

        for (UploadedPackage uploadedPackage : uploadedPackageList){
            if (uploadedPackage.getPackageName().equals(packageName1)){
                assertEquals(application1, uploadedPackage.getApplication());
                assertEquals(EMPTY_STRING, uploadedPackage.getTechnology());
                assertEquals(EMPTY_STRING, uploadedPackage.getPhase());
                assertEquals(EMPTY_STRING, uploadedPackage.getScenarioName());
                assertEquals(EMPTY_STRING, uploadedPackage.getConnection());
            } else if (uploadedPackage.getPackageName().equals(packageName2)){
                assertEquals(EMPTY_STRING, uploadedPackage.getApplication());
                assertEquals(technology2, uploadedPackage.getTechnology());
                assertEquals(phase2, uploadedPackage.getPhase());
                assertEquals(scenarioName2, uploadedPackage.getScenarioName());
                assertEquals(connection2, uploadedPackage.getConnection());
            } else {
                fail();
            }
        }
    }

}
