package DAOTests;

import java.time.Instant;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageUploadDAO;

import static org.junit.Assert.assertEquals;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class PackageUploadDAOTests {

    @Autowired
    private ConnectionsDAO connectionsDAO;

    @Autowired
    private PackageUploadDAO packageUploadDAO;

    @Autowired
    private ErrorMetadataDAO errorMetadataDAO;

    @Autowired
    private LogDAO logDAO;

    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";

    private String technology = "Oracle";
    private String phase = "Analysis";
    private String scenarioName = "Oracle DDL Dataflow Analysis";

    private static String errorLogType = "ERROR";
    private static String debugLogType = "DEBUG";

    private static String threadName = "main";
    private static int threadId = 1;
    private static String errorLevel = "ERROR";
    private static String debugLevel = "INFO";
    private static String loggerName = "test.logger";
    private static long epochSecond = Instant.now().getEpochSecond();
    private static long nanoSecond = Instant.now().getNano();
    private static String message = "Test message";
    private static String exceptionName = "NullPointerException";
    private static String exceptionMessage = "Thrown null ptr exception.";

    private String application = "Test Application";

    private Long connDbId;
    private Long errorId;

    private final Long ONE = new Long(1);

    private String packageName = "My Package 1";

    @Before
    public void before(){
        connDbId = connectionsDAO.getIdByData(new Connection(null, connectionId, executionId, null));
        errorId = errorMetadataDAO.getAllIds().get(0);
    }

    @Test
    public void save_createNewPackageForApplication_success(){
        UploadedPackage uploadedPackage = new UploadedPackage(
                "My Package 3",
                application
        );

        Long inserted = packageUploadDAO.save(uploadedPackage);
        assertEquals(ONE, inserted);
    }

    @Test
    public void save_createNewPackageForConnection_success(){
        UploadedPackage uploadedPackage = new UploadedPackage(
                "My Package 3",
                technology,
                phase,
                scenarioName,
                connectionId
        );

        Long inserted = packageUploadDAO.save(uploadedPackage);
        assertEquals(ONE, inserted);
    }

    @Test
    public void uploadDebug_insertNewDebugLogBelongingToAPackage_oneLogInserted(){
        UploadedLog uploadedLog = new UploadedLog(
                debugLogType, application, "", threadName, threadId, loggerName, 0, debugLevel,
                epochSecond, nanoSecond, message, exceptionName, exceptionMessage, 0L
        );

        Long inserted = packageUploadDAO.upload(uploadedLog, packageName);
        assertEquals(ONE, inserted);

        DebugLog debugLog = (DebugLog) logDAO.getAll().get(0);

        assertEquals(debugLog.getType(), uploadedLog.getType());
        assertEquals(debugLog.getMessage(), uploadedLog.getMessage());
        assertEquals(debugLog.getApplication(), uploadedLog.getApplication());
        assertEquals(debugLog.getContext(), uploadedLog.getContext());
        assertEquals(debugLog.getEpochSecond(), uploadedLog.getEpochSecond());
        assertEquals(debugLog.getExceptionName(), uploadedLog.getExceptionName());
        assertEquals(debugLog.getExtendedStacktrace(), uploadedLog.getExtendedStacktrace());
        assertEquals(debugLog.getLoggerName(), uploadedLog.getLoggerName());
        assertEquals(debugLog.getLevel(), uploadedLog.getLogLevel());
        assertEquals(debugLog.getNanoSecond(), uploadedLog.getNanoOfSecond());
        assertEquals(debugLog.getThreadId(), uploadedLog.getThreadId());
        assertEquals(debugLog.getWorkerNumber(), uploadedLog.getWorkerNumber());
    }

    @Test
    public void uploadDebug_insertNewErrorLogBelongingToAPackage_oneLogInserted(){
        UploadedLog uploadedLog = new UploadedLog(
                errorLogType, application, "", threadName, threadId, loggerName, 0, errorLevel,
                epochSecond, nanoSecond, null, exceptionName, exceptionMessage, 0L
        );

        Long inserted = packageUploadDAO.upload(uploadedLog, errorId, packageName);
        assertEquals(ONE, inserted);

        ErrorLog errorLog = (ErrorLog) logDAO.getAll().get(0);

        assertEquals(errorLog.getType(), uploadedLog.getType());
        assertEquals(errorLog.getMessage(), uploadedLog.getMessage());
        assertEquals(errorLog.getApplication(), uploadedLog.getApplication());
        assertEquals(errorLog.getContext(), uploadedLog.getContext());
        assertEquals(errorLog.getEpochSecond(), uploadedLog.getEpochSecond());
        assertEquals(errorLog.getExceptionName(), uploadedLog.getExceptionName());
        assertEquals(errorLog.getExtendedStacktrace(), uploadedLog.getExtendedStacktrace());
        assertEquals(errorLog.getLoggerName(), uploadedLog.getLoggerName());
        assertEquals(errorLog.getLevel(), uploadedLog.getLogLevel());
        assertEquals(errorLog.getNanoSecond(), uploadedLog.getNanoOfSecond());
        assertEquals(errorLog.getThreadId(), uploadedLog.getThreadId());
        assertEquals(errorLog.getWorkerNumber(), uploadedLog.getWorkerNumber());
    }

}
