package DAOTests;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PruneCheckingDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;

import static utils.Utils.getDebugLog;
import static utils.Utils.getErrorLog;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class PruneCheckingDAOTests {

    @Autowired
    private PruneCheckingDAO pruneCheckingDAO;

    @Autowired
    private WorkflowExecutionDAO workflowExecutionDAO;

    @Autowired
    private LogDAO logDAO;

    @Autowired
    private ConnectionsDAO connectionsDAO;

    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";
    private String application = "Test Application";

    private final Long ONE = new Long(1);

    private Long connDbId;

    @Before
    public void before(){
        connDbId = connectionsDAO.getIdByData(new Connection(null, connectionId, executionId, null));
    }


    @Test
    public void amountOfPotentiallyDeletedWorkflows_checkIfTestDataWorkflowWouldBeDeleted_oneReturned(){
        List<WorkflowExecution> deletedWorkflows = pruneCheckingDAO.getPotentiallyDeletedWorkflows(0); // 0 means deleted automatically
        assertEquals(ONE, new Long(deletedWorkflows.size()));
    }

    @Test
    public void amountOfPotentiallyDeletedWorkflowLogs_insertFiveLogsForATestDataWorkflowAndCheckIfTheyWouldBeDeleted_fiveReturned(){
        for (int i = 0; i < 5; ++i){
            DebugLog debugLog = getDebugLog("WE1", "SE1", "OracleTestDB");
            Long inserted = logDAO.save(debugLog, connDbId);
            assertEquals(ONE, inserted);
        }

        Optional<WorkflowExecution> workflowExecutionOptional = workflowExecutionDAO.find(workflowId);
        assertTrue(workflowExecutionOptional.isPresent());
        WorkflowExecution workflowExecution = workflowExecutionOptional.get();

        Integer deletedWorkflowLogs = pruneCheckingDAO.getAmountOfPotentiallyDeletedWorkflowLogs(0, Arrays.asList(workflowExecution)); // 0 means deleted automatically
        assertEquals(new Integer(5), deletedWorkflowLogs);
    }

    @Test
    public void amountOfPotentiallyDeletedLogs_insertFiveLogsForATestDataWorkflowAndFiveThatAreNotCheckIfTheyWouldBeDeleted_fiveReturned(){
        for (int i = 0; i < 5; ++i){
            DebugLog debugLog = getDebugLog("WE1", "SE1", "OracleTestDB");
            Long inserted = logDAO.save(debugLog, connDbId);
            assertEquals(ONE, inserted);
        }

        for (int i = 0; i < 5; ++i){
            DebugLog debugLog = getDebugLog(null, null, null);
            Long inserted = logDAO.save(debugLog, null);
            assertEquals(ONE, inserted);
        }

        Integer deletedLogs = pruneCheckingDAO.getAmountOfPotentiallyDeletedLogs(0); // 0 means deleted automatically
        assertEquals(new Integer(5), deletedLogs);
    }

}
