package DAOTests;

import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.DistinctErrorType;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.RepositoryMetadataDAO;

import static utils.Utils.getDebugLog;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class RepositoryMetadataDAOTests {

    @Autowired
    private RepositoryMetadataDAO repositoryMetadataDAO;

    @Autowired
    private ConnectionsDAO connectionsDAO;

    @Autowired
    private LogDAO logDAO;

    private Long errorId = null;
    private Long connDbId = null;
    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";

    private final Long ONE = 1L;

    @Before
    public void before(){
        connDbId = connectionsDAO.getIdByData(new Connection(null, connectionId, executionId, null));
    }

    @Test
    public void getDistinctApplications_insertThreeLogsTwoDistinctApplications_twoReturned(){
        String FIRST_APPLICATION = "FIRST APPLICATION";
        String SECOND_APPLICATION = "SECOND APPLICATION";

        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, FIRST_APPLICATION);
        Long insertedRows1 = logDAO.save(debugLog1, connDbId);
        assertEquals(ONE, insertedRows1);

        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, SECOND_APPLICATION);
        Long insertedRows2 = logDAO.save(debugLog2, connDbId);
        assertEquals(ONE, insertedRows2);

        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, FIRST_APPLICATION);
        Long insertedRows3 = logDAO.save(debugLog3, connDbId);
        assertEquals(ONE, insertedRows3);

        List<String> distinctApplications = repositoryMetadataDAO.getDistinctApplications();
        assertEquals(2, distinctApplications.size());
        assertTrue(distinctApplications.contains(FIRST_APPLICATION));
        assertTrue(distinctApplications.contains(SECOND_APPLICATION));
    }

    @Test
    public void getDistinctErrorTypes_distinctTestDataErrorTypes_oneReturned(){
        Filter filter = new Filter.FilterBuilder().buildFilter();
        List<DistinctErrorType> distinctErrorTypes = repositoryMetadataDAO.getDistinctErrorTypes(filter);
        assertEquals(ONE, new Long(distinctErrorTypes.size()));
    }

    @Test
    public void getDistinctErrorTypes_distinctTestDataErrorTypesFilteredOut_noneReturned(){
        Filter.FilterBuilder filterBuilder = new Filter.FilterBuilder();
        filterBuilder.setCategories(Arrays.asList("SOME CATEGORY THAT IS NOT THERE"));
        Filter filter = filterBuilder.buildFilter();

        List<DistinctErrorType> distinctErrorTypes = repositoryMetadataDAO.getDistinctErrorTypes(filter);
        assertEquals(0, distinctErrorTypes.size());
    }

    @Test
    public void getDistinctCategories_distinctTestDataCategories_oneReturned(){
        Filter filter = new Filter.FilterBuilder().buildFilter();
        List<String> distinctCategories = repositoryMetadataDAO.getDistinctCategories(filter);
        assertEquals(ONE, new Long(distinctCategories.size()));
    }

    @Test
    public void getDistinctCategories_distinctTestDataCategoriesFilteredOut_noneReturned(){
        Filter.FilterBuilder filterBuilder = new Filter.FilterBuilder();
        filterBuilder.setWorkflowExecutionId("SOME EXECUTION THAT IS NOT THERE");
        Filter filter = filterBuilder.buildFilter();

        List<String> distinctCategories = repositoryMetadataDAO.getDistinctCategories(filter);
        assertEquals(0, distinctCategories.size());
    }

    @Test
    public void getConnectionsForScenario_getConnectionsForAGivenScenario_oneReturned(){
        List<String> connectionNames = repositoryMetadataDAO.getConnectionsForScenario(executionId);
        assertEquals(ONE, new Long(connectionNames.size()));
    }

}
