package DAOTests;

import java.util.List;
import java.util.Optional;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class WorkflowExecutionDAOTests {

    @Autowired
    private WorkflowExecutionDAO workflowExecutionDAO;

    private String workflowId = "WE1"; // from test_data.sql

    private final Long ONE = 1L;

    @Test
    public void save_addNewWorkflowExecution_added(){
        WorkflowExecution workflowExecution = new WorkflowExecution("WE2");

        Long insertedRows = workflowExecutionDAO.save(workflowExecution);
        assertEquals(ONE, insertedRows);

        Optional<WorkflowExecution> foundWorkflowExecutionOpt = workflowExecutionDAO.find(workflowId);
        assertTrue(foundWorkflowExecutionOpt.isPresent());
    }

    @Test(expected = DuplicateKeyException.class)
    public void save_addNewWorkflowExecutionWithExistingID_shouldThrowException(){
        WorkflowExecution workflowExecution = new WorkflowExecution(workflowId);
        Long insertedRows = workflowExecutionDAO.save(workflowExecution);
    }

    @Test
    public void delete_deleteWorkflow_deleted(){
        WorkflowExecution workflowExecution = new WorkflowExecution(workflowId);
        Long deletedRows = workflowExecutionDAO.delete(workflowExecution);
        assertEquals(ONE, deletedRows);
    }

    @Test
    public void find_findWorkflowExecution_found(){
        WorkflowExecution workflowExecution = new WorkflowExecution(workflowId);

        Optional<WorkflowExecution> foundWorkflowExecutionOpt = workflowExecutionDAO.find(workflowId);
        assertTrue(foundWorkflowExecutionOpt.isPresent());

        WorkflowExecution foundExecution = foundWorkflowExecutionOpt.get();

        assertEquals(workflowExecution.getWorkflowExecutionId(), foundExecution.getWorkflowExecutionId());
    }

    @Test
    public void getAll_getAllWorkflowExecutions_oneFound(){
        List<WorkflowExecution> workflowExecutionList = workflowExecutionDAO.getAll();
        assertEquals(ONE, new Long(workflowExecutionList.size()));

        WorkflowExecution workflowExecution = workflowExecutionList.get(0);

        assertEquals(workflowId, workflowExecution.getWorkflowExecutionId());
    }

}
