package ServiceTests;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import TestConfig.ServiceTestsConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ScenarioExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.ForcePruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.WorkflowDoesNotExistException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.ForcePruneServiceImpl;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {ServiceTestsConfiguration.class, ForcePruneServiceTests.ForcePruneServiceConfig.class})
@ActiveProfiles({"test"})
public class ForcePruneServiceTests {

    @Configuration
    static class ForcePruneServiceConfig {
        @Bean
        public ForcePruneService forcePruneService(){
            return new ForcePruneServiceImpl(workflowExecutionDAO, logDAO, pruneService, errorDefinitionDAO,
                    errorMetadataDAO, scenarioExecutionDAO, connectionsDAO,packageDAO);
        }
    }

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Mock
    private static ErrorDefinitionDAO errorDefinitionDAO;

    @Mock
    private static ErrorMetadataDAO errorMetadataDAO;

    @Mock
    private static PruneService pruneService;

    @Mock
    private static ScenarioExecutionDAO scenarioExecutionDAO;

    @Mock
    private static ConnectionsDAO connectionsDAO;

    @Mock
    private static PackageDAO packageDAO;

    @Mock
    private static WorkflowExecutionDAO workflowExecutionDAO;

    @Mock
    private static LogDAO logDAO;

    @Autowired
    @InjectMocks
    private ForcePruneService forcePruneService;

    @Test
    public void forcePruneWorkflowExecution_passExistingWorkflowExecution_workflowDeletedErrorsChecked(){
        Optional<WorkflowExecution> workflowExecutionOptional = Optional.of(new WorkflowExecution("WE1"));
        when(workflowExecutionDAO.find(anyString())).thenReturn(workflowExecutionOptional);

        List<Long> allIds = Arrays.asList(1L, 2L, 3L);
        List<Long> usedIds = Arrays.asList(1L, 2L);

        when(errorMetadataDAO.getAllIds()).thenReturn(allIds);
        when(errorMetadataDAO.getDistinctIds()).thenReturn(usedIds);

        when(errorDefinitionDAO.getAll()).thenReturn(allIds);
        when(errorDefinitionDAO.getDistinct()).thenReturn(usedIds);

        forcePruneService.forcePruneWorkflow("WE1");

        verify(pruneService, times(1)).dropPruningFor(workflowExecutionOptional.get());
        verify(workflowExecutionDAO, times(1)).delete(workflowExecutionOptional.get());
        verify(errorMetadataDAO, times(1)).delete(Arrays.asList(3L));
        verify(errorDefinitionDAO, times(1)).delete(Arrays.asList(3L));
    }

    @Test(expected = WorkflowDoesNotExistException.class)
    public void forcePruneWorkflowExecution_passEmptyOptional_exceptionThrown(){
        Optional<WorkflowExecution> workflowExecutionOptional = Optional.empty();
        when(workflowExecutionDAO.find(anyString())).thenReturn(workflowExecutionOptional);
        forcePruneService.forcePruneWorkflow("WE1");

        verify(pruneService, never()).dropPruningFor(workflowExecutionOptional.get());
        verify(workflowExecutionDAO, never()).delete(workflowExecutionOptional.get());
    }



}
