package ServiceTests;

import java.util.Arrays;
import java.util.List;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import TestConfig.ServiceTestsConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.FullTextSearchService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.FullTextSearchServiceImpl;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {ServiceTestsConfiguration.class, FullTextSearchServiceTests.FullTextConfiguration.class})
@ActiveProfiles({"test"})
public class FullTextSearchServiceTests {

    @Configuration
    static class FullTextConfiguration {
        @Bean
        public FullTextSearchService fullTextSearchService(){
            return new FullTextSearchServiceImpl(logDAO);
        }
    }

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);;

    @Mock
    private static LogDAO logDAO;

    @Autowired
    @InjectMocks
    private FullTextSearchService fullTextSearchService;

    private String errorCategory = "TEST_CATEGORY";
    private String errorType = "TEST_ERROR";
    private String userMessage = "Test user message.";
    private String technicalMessage = "Test technical message.";
    private String solution = "Test solution.";
    private String impact = "Test impact.";
    private String logLevel = "Test log level.";
    private String application = "Test Application";

    @Test
    public void fullText_mockFullTextCallReturnBasicErrorMetadataCheckCorrectDTO_success(){
        ErrorMetadata errorMetadata = new ErrorMetadata(
                application,
                errorCategory,
                errorType,
                userMessage,
                technicalMessage,
                solution,
                impact,
                logLevel
        );

        when(logDAO.fullText(any(), any())).thenReturn(Arrays.asList(errorMetadata));

        Filter filter = new Filter.FilterBuilder().buildFilter();

        List<ErrorDTO> errors = fullTextSearchService.fullText("", filter);
        assertEquals(1, errors.size());

        ErrorDTO errorDTO = errors.get(0);

        assertEquals(errorMetadata.getErrorDefinition().getErrorCategory(), errorDTO.getErrorCategory());
        assertEquals(errorMetadata.getErrorDefinition().getErrorType(), errorDTO.getErrorType());
        assertEquals(errorMetadata.getErrorDefinition().getUserMessage(), errorDTO.getUserMessage());
        assertEquals(errorMetadata.getErrorDefinition().getTechnicalMessage(), errorDTO.getTechnicalMessage());
        assertEquals(errorMetadata.getErrorDefinition().getSolution(), errorDTO.getSolution());
        assertEquals(errorMetadata.getErrorDefinition().getImpact(), errorDTO.getImpact());
        assertEquals(errorMetadata.getErrorDefinition().getLogLevel(), errorDTO.getLogLevel());
        assertEquals(errorMetadata.getApplication(), errorDTO.getApplication());
    }

}
