package ServiceTests;

import java.util.Arrays;
import java.util.List;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import TestConfig.ServiceTestsConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.LogMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogContextService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.LogContextServiceImpl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.when;
import static utils.Utils.getDebugLog;
import static utils.Utils.getErrorLog;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {ServiceTestsConfiguration.class, LogContextServiceTests.LogContextServiceConfiguration.class})
@ActiveProfiles({"test"})
public class LogContextServiceTests {

    @Configuration
    static class LogContextServiceConfiguration {
        @Bean
        public LogContextService logContextService(){
            return new LogContextServiceImpl(logDAO);
        }
    }

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Mock
    private static LogDAO logDAO;

    @Autowired
    @InjectMocks
    private LogContextService logContextService;

    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";
    private String application = "Test Application";

    @Test
    public void getContextForLog_returnDataFromMocksCheckIfReturnedContextCorrectOrder_success(){
        LogMetadata logMetadata = new LogMetadata(
                "ERROR", 1L, workflowId, executionId, null
        );

        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, application, 1, 1);
        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, application, 2, 1);
        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, application, 3, 1);
        DebugLog debugLog4 = getDebugLog(workflowId, executionId, connectionId, application, 4, 1);
        DebugLog debugLog5 = getDebugLog(workflowId, executionId, connectionId, application, 5, 1);

        List<Log> earlierLogs = Arrays.asList(debugLog5, debugLog4, debugLog3, debugLog2, debugLog1);

        ErrorLog rootLog = getErrorLog(workflowId, executionId, connectionId, application, 5, 2);

        DebugLog debugLog6 = getDebugLog(workflowId, executionId, connectionId, application, 6, 1);
        DebugLog debugLog7 = getDebugLog(workflowId, executionId, connectionId, application, 7, 1);
        DebugLog debugLog8 = getDebugLog(workflowId, executionId, connectionId, application, 8, 1);
        DebugLog debugLog9 = getDebugLog(workflowId, executionId, connectionId, application, 9, 1);
        DebugLog debugLog10 = getDebugLog(workflowId, executionId, connectionId, application, 10, 1);

        List<Log> laterLogs = Arrays.asList(debugLog6, debugLog7, debugLog8, debugLog9, debugLog10);

        when(logDAO.getLogMetadata(any())).thenReturn(logMetadata);
        when(logDAO.find(any(), any())).thenReturn(rootLog);
        when(logDAO.loadEarlier(any(Log.class), any(Long.class), anyLong(), anyLong(), any(LogMetadata.class))).thenReturn(earlierLogs);
        when(logDAO.loadLater(any(Log.class), any(Long.class), anyLong(), anyLong(), any(LogMetadata.class))).thenReturn(laterLogs);

        List<? extends Log> logs = logContextService.getContextForLog(0L);

        assertEquals(11, logs.size());

        // check order
        Log last = null;
        for (Log log : logs){
            if (last == null){
                last = log;
                continue;
            }

            if (last.getEpochSecond() < log.getEpochSecond()){
                last = log;
            } else if (last.getEpochSecond() == log.getEpochSecond() && last.getNanoSecond() < log.getNanoSecond()){
                last = log;
            } else {
                fail();
            }
        }
    }

}
