package utils;

import java.time.Instant;
import java.util.Map;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.LoggedExceptionData;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.additionalcontext.BaseContextMap;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.additionalcontext.ErrorContextMap;

import static org.junit.Assert.assertEquals;

public class Utils {

    private static String application = "Test Application";

    private static String errorCategory = "TEST_CATEGORY";
    private static String errorType = "TEST_ERROR";
    private static String userMessage = "Test user message.";
    private static String technicalMessage = "Test technical message.";
    private static String solution = "Test solution.";
    private static String impact = "Test impact.";
    private static String logLevel = "ERROR";

    private static String errorLogType = "ERROR";
    private static String debugLogType = "DEBUG";

    private static String threadName = "main";
    private static int threadId = 1;
    private static String errorLevel = "ERROR";
    private static String debugLevel = "INFO";
    private static String loggerName = "test.logger";
    private static long epochSecond = Instant.now().getEpochSecond();
    private static long nanoSecond = Instant.now().getNano();
    private static String message = "Test message";
    private static String exceptionName = "NullPointerException";
    private static String exceptionMessage = "Thrown null ptr exception.";

    private static String technology = "Oracle";
    private static String phase = "Analysis";
    private static String scenarioName = "Oracle DDL Dataflow Analysis";

    private static final String EMPTY_STRING = "";

    public static ErrorLog getErrorLog(String workflowId, String executionId, String connectionId, String application){
        return getErrorLog(workflowId, executionId, connectionId, application, epochSecond, nanoSecond);
    }

    public static ErrorLog getErrorLog(String workflowId, String executionId, String connectionId){
        return getErrorLog(workflowId, executionId, connectionId, application, epochSecond, nanoSecond);
    }

    public static ErrorLog getErrorLog(String workflowId, String executionId, String connectionId, String application, long epochSecond, long nanoSecond){
        ErrorContextMap errorContextMap;

        if (workflowId != null && executionId != null){
            errorContextMap = new ErrorContextMap(
                    workflowId, executionId, technology, connectionId, phase, scenarioName, EMPTY_STRING, 0, userMessage, technicalMessage, solution,
                    impact, errorCategory, errorType
            );
        } else {
            errorContextMap = new ErrorContextMap(
                    workflowId, null, null, connectionId, null, null, EMPTY_STRING, 0, userMessage, technicalMessage, solution,
                    impact, errorCategory, errorType
            );
        }

        return new ErrorLog(
                errorLogType, threadName, threadId, errorLevel, loggerName, new eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Instant(epochSecond, nanoSecond),
                null, new LoggedExceptionData(exceptionName, exceptionMessage), application, errorContextMap
        );
    }

    public static ErrorLog getPackageErrorLog(String packageName){
        ErrorContextMap errorContextMap = new ErrorContextMap(
                null, null, null, null, null, null, EMPTY_STRING, 0, userMessage, technicalMessage, solution,
                impact, errorCategory, errorType, packageName
        );

        return new ErrorLog(
                errorLogType, threadName, threadId, errorLevel, loggerName, new eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Instant(epochSecond, nanoSecond),
                null, new LoggedExceptionData(exceptionName, exceptionMessage), application, errorContextMap
        );
    }

    public static ErrorLog getNonWorkflowErrorLog(){
        ErrorContextMap errorContextMap = new ErrorContextMap(
                null, null, null, null, null, null, EMPTY_STRING, 0, userMessage, technicalMessage, solution,
                impact, errorCategory, errorType
        );

        return new ErrorLog(
                errorLogType, threadName, threadId, errorLevel, loggerName, new eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Instant(epochSecond, nanoSecond),
                null, new LoggedExceptionData(exceptionName, exceptionMessage), application, errorContextMap
        );
    }

    public static DebugLog getDebugLog(String workflowId, String executionId, String connectionId, String application){
        return getDebugLog(workflowId, executionId, connectionId, application, epochSecond, nanoSecond);
    }

    public static DebugLog getDebugLog(String workflowId, String executionId, String connectionId){
        return getDebugLog(workflowId, executionId, connectionId, application, epochSecond, nanoSecond);
    }

    public static DebugLog getDebugLog(String workflowId, String executionId, String connectionId, String application, long epochSecond, long nanoSecond){
        BaseContextMap baseContextMap;

        if (workflowId != null && executionId != null){
            baseContextMap = new BaseContextMap(
                    workflowId, executionId, technology, phase, scenarioName, connectionId, EMPTY_STRING, 0);
        } else {
            baseContextMap = new BaseContextMap(
                    workflowId, null, null, null, null, connectionId, EMPTY_STRING, 0);
        }

        return new DebugLog(
                debugLogType, threadName, threadId, debugLevel, loggerName, new eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Instant(epochSecond, nanoSecond),
                message, null, application, baseContextMap);
    }

    public static void checkErrorLogEquals(ErrorLog expected, ErrorLog other){
        assertEquals(expected.getType(), other.getType());
        assertEquals(expected.getThread(), other.getThread());
        assertEquals(expected.getThreadId(), other.getThreadId());
        assertEquals(expected.getLevel(), other.getLevel());
        assertEquals(expected.getLoggerName(), other.getLoggerName());
        assertEquals(expected.getEpochSecond(), other.getEpochSecond());
        assertEquals(expected.getNanoSecond(), other.getNanoSecond());
        assertEquals(expected.getMessage(), other.getMessage());
        assertEquals(expected.getExceptionName(), other.getExceptionName());
        assertEquals(expected.getExtendedStacktrace(), other.getExtendedStacktrace());
        assertEquals(expected.getApplication(), other.getApplication());
        assertEquals(expected.getWorkflowExecutionId(), other.getWorkflowExecutionId());
        assertEquals(expected.getScenarioExecutionId(), other.getScenarioExecutionId());
        assertEquals(expected.getTechnology(), other.getTechnology());
        assertEquals(expected.getConnectionId(), other.getConnectionId());
        assertEquals(expected.getPhase(), other.getPhase());
        assertEquals(expected.getScenario(), other.getScenario());
        assertEquals(expected.getContext(), other.getContext());
        assertEquals(expected.getWorkerNumber(), other.getWorkerNumber());
        assertEquals(expected.getUserMessage(), other.getUserMessage());
        assertEquals(expected.getTechnicalMessage(), other.getTechnicalMessage());
        assertEquals(expected.getSolution(), other.getSolution());
        assertEquals(expected.getLineageImpact(), other.getLineageImpact());
        assertEquals(expected.getCategory(), other.getCategory());
        assertEquals(expected.getErrorType(), other.getErrorType());
    }

    public static void checkDebugLogEquals(DebugLog expected, DebugLog other){
        assertEquals(expected.getType(), other.getType());
        assertEquals(expected.getThread(), other.getThread());
        assertEquals(expected.getThreadId(), other.getThreadId());
        assertEquals(expected.getLevel(), other.getLevel());
        assertEquals(expected.getLoggerName(), other.getLoggerName());
        assertEquals(expected.getEpochSecond(), other.getEpochSecond());
        assertEquals(expected.getNanoSecond(), other.getNanoSecond());
        assertEquals(expected.getMessage(), other.getMessage());
        assertEquals(expected.getExceptionName(), other.getExceptionName());
        assertEquals(expected.getExtendedStacktrace(), other.getExtendedStacktrace());
        assertEquals(expected.getApplication(), other.getApplication());
        assertEquals(expected.getWorkflowExecutionId(), other.getWorkflowExecutionId());
        assertEquals(expected.getScenarioExecutionId(), other.getScenarioExecutionId());
        assertEquals(expected.getTechnology(), other.getTechnology());
        assertEquals(expected.getConnectionId(), other.getConnectionId());
        assertEquals(expected.getPhase(), other.getPhase());
        assertEquals(expected.getScenario(), other.getScenario());
        assertEquals(expected.getContext(), other.getContext());
        assertEquals(expected.getWorkerNumber(), other.getWorkerNumber());
    }

    public static void checkDebugLogEquals(DebugLog debugLog, Map<String, Object> retrievedLog){
        assertEquals(debugLog.getType(), retrievedLog.get("type"));
        assertEquals(debugLog.getThread(), retrievedLog.get("thread"));
        assertEquals(debugLog.getThreadId(), retrievedLog.get("threadId"));
        assertEquals(debugLog.getLevel(), retrievedLog.get("level"));
        assertEquals(debugLog.getLoggerName(), retrievedLog.get("loggerName"));
        assertEquals(debugLog.getEpochSecond(), ((Integer) retrievedLog.get("epochSecond")).longValue());
        assertEquals(debugLog.getNanoSecond(), ((Integer) retrievedLog.get("nanoSecond")).longValue());
        assertEquals(debugLog.getMessage(), retrievedLog.get("message"));
        assertEquals(debugLog.getExceptionName(), retrievedLog.get("exceptionName"));
        assertEquals(debugLog.getExtendedStacktrace(), retrievedLog.get("extendedStacktrace"));
        assertEquals(debugLog.getApplication(), retrievedLog.get("application"));
        assertEquals(debugLog.getWorkflowExecutionId(), retrievedLog.get("workflowExecutionId"));
        assertEquals(debugLog.getScenarioExecutionId(), retrievedLog.get("scenarioExecutionId"));
        assertEquals(debugLog.getTechnology(), retrievedLog.get("technology"));
        assertEquals(debugLog.getConnectionId(), retrievedLog.get("connectionId"));
        assertEquals(debugLog.getPhase(), retrievedLog.get("phase"));
        assertEquals(debugLog.getScenario(), retrievedLog.get("scenario"));
        assertEquals(debugLog.getContext(), retrievedLog.get("context"));
        assertEquals(debugLog.getWorkerNumber(), retrievedLog.get("workerNumber"));
    }

    public static void checkErrorLogEquals(ErrorLog errorLog, Map<String, Object> retrievedLog){
        assertEquals(errorLog.getType(), retrievedLog.get("type"));
        assertEquals(errorLog.getThread(), retrievedLog.get("thread"));
        assertEquals(errorLog.getThreadId(), retrievedLog.get("threadId"));
        assertEquals(errorLog.getLevel(), retrievedLog.get("level"));
        assertEquals(errorLog.getLoggerName(), retrievedLog.get("loggerName"));
        assertEquals(errorLog.getEpochSecond(), ((Integer) retrievedLog.get("epochSecond")).longValue());
        assertEquals(errorLog.getNanoSecond(), ((Integer) retrievedLog.get("nanoSecond")).longValue());
        assertEquals(errorLog.getMessage(), retrievedLog.get("message"));
        assertEquals(errorLog.getExceptionName(), retrievedLog.get("exceptionName"));
        assertEquals(errorLog.getExtendedStacktrace(), retrievedLog.get("extendedStacktrace"));
        assertEquals(errorLog.getApplication(), retrievedLog.get("application"));
        assertEquals(errorLog.getWorkflowExecutionId(), retrievedLog.get("workflowExecutionId"));
        assertEquals(errorLog.getScenarioExecutionId(), retrievedLog.get("scenarioExecutionId"));
        assertEquals(errorLog.getTechnology(), retrievedLog.get("technology"));
        assertEquals(errorLog.getConnectionId(), retrievedLog.get("connectionId"));
        assertEquals(errorLog.getPhase(), retrievedLog.get("phase"));
        assertEquals(errorLog.getScenario(), retrievedLog.get("scenario"));
        assertEquals(errorLog.getContext(), retrievedLog.get("context"));
        assertEquals(errorLog.getWorkerNumber(), retrievedLog.get("workerNumber"));
        assertEquals(errorLog.getUserMessage(), retrievedLog.get("userMessage"));
        assertEquals(errorLog.getTechnicalMessage(), retrievedLog.get("technicalMessage"));
        assertEquals(errorLog.getSolution(), retrievedLog.get("solution"));
        assertEquals(errorLog.getLineageImpact(), retrievedLog.get("lineageImpact"));
        assertEquals(errorLog.getCategory(), retrievedLog.get("category"));
        assertEquals(errorLog.getErrorType(), retrievedLog.get("errorType"));
    }

    public static void checkErrorMetadataEquals(ErrorLog errorLog, ErrorMetadata errorMetadata){
        assertEquals(errorLog.getCategory(), errorMetadata.getErrorDefinition().getErrorCategory());
        assertEquals(errorLog.getErrorType(), errorMetadata.getErrorDefinition().getErrorType());
        assertEquals(errorLog.getUserMessage(), errorMetadata.getErrorDefinition().getUserMessage());
        assertEquals(errorLog.getTechnicalMessage(), errorMetadata.getErrorDefinition().getTechnicalMessage());
        assertEquals(errorLog.getSolution(), errorMetadata.getErrorDefinition().getSolution());
        assertEquals(errorLog.getLineageImpact(), errorMetadata.getErrorDefinition().getImpact());
        assertEquals(logLevel, errorMetadata.getErrorDefinition().getLogLevel());
        assertEquals(errorLog.getApplication(), errorMetadata.getApplication());
    }

}
