package eu.profinit.manta.bachelor.thesis.log.viewer.broker.consumer;

import java.io.IOException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.ObjectMapper;

import eu.profinit.manta.bachelor.thesis.log.viewer.errors.Categories;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;

/**
 * Implementation of the log resolver.
 * @author jkovac
 */
@Component
public class LogResolverImpl implements LogResolver {

    /**
     * Logger
     */
    private Logger logger = new Logger(getClass());

    /**
     * Jackson object mapper
     */
    private ObjectMapper objectMapper;

    /**
     * Instantiates a new Log resolver.
     *
     * @param objectMapper Jackson object mapper
     */
    @Autowired
    public LogResolverImpl(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    @Override
    public Log resolve(String jsonLog) throws UnableToResolveException {
        Log log;
        try {
            log = objectMapper.readValue(jsonLog, Log.class);
        } catch (IOException e) {
            logger.log(Categories.resolverErrors().unableToResolveLog().catching(e));
            throw new UnableToResolveException(e);
        }
        return log;
    }

}
