package eu.profinit.manta.bachelor.thesis.log.viewer.broker.listener;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

import eu.profinit.manta.bachelor.thesis.log.viewer.broker.consumer.LogResolver;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.consumer.UnableToResolveException;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.Handleable;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.MessageHandler;

/**
 * Class implementing listener to the designated logging queue.
 * @author jkovac
 */
@Component
public class LogListener {

    /**
     * Resolver parsing logs from the JSON messages
     */
    private LogResolver logResolver;

    /**
     * Handler handling logs based on their type
     */
    private MessageHandler handler;

    @Autowired
    public LogListener(LogResolver logResolver, MessageHandler handler) {
        this.logResolver = logResolver;
        this.handler = handler;
    }

    /**
     * JMS listener to the designated logging queue
     * @param message message polled from the JMS queue
     */
    @JmsListener(destination = "#{@consumerConfig.loggingQueue}")
    public void handleMessage(String message){
        Handleable log = null;

        try {
            log = logResolver.resolve(message);
        } catch (UnableToResolveException e) {
            return;
        }

        log.handle(handler);
    }

}
