package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import java.util.Collections;
import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;

/**
 * Common ancestor for controllers requiring filtering.
 *
 * @author jkovac
 */
public abstract class LogViewerFilterBasedController extends LogViewerBaseController {

    /**
     * empty string
     */
    protected static final String EMPTY_STRING = "";


    /**
     * Create filter.
     *
     * @param workflowExecutionId the workflow execution id
     * @param scenarioExecutionId the scenario execution id
     * @param connectionId        the connection id
     * @param packageName         the package name
     * @param categories          the categories
     * @param errorTypes          the error types
     * @param applications        the applications
     * @param offset              the offset
     * @return the filter
     */
    protected Filter createFilter(String workflowExecutionId,
                                String scenarioExecutionId,
                                String connectionId,
                                String packageName,
                                List<String> categories,
                                List<String> errorTypes,
                                List<String> applications,
                                Long offset){

        Filter.FilterBuilder filterBuilder = new Filter.FilterBuilder();

        filterBuilder.setWorkflowExecutionId(workflowExecutionId);
        filterBuilder.setScenarioExecutionId(scenarioExecutionId);
        filterBuilder.setConnectionId(connectionId);
        filterBuilder.setPackageName(packageName);
        filterBuilder.setCategories(categories);
        filterBuilder.setErrorTypes(errorTypes);
        filterBuilder.setApplications(applications);
        filterBuilder.setOffset(offset);

        return filterBuilder.buildFilter();
    }

}
