package eu.profinit.manta.bachelor.thesis.log.viewer.controllers.configuration;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Optional;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.multipart.commons.CommonsMultipartResolver;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.BasicAuth;
import springfox.documentation.service.SecurityScheme;
import springfox.documentation.service.Tag;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import static springfox.documentation.builders.PathSelectors.regex;

/**
 * Configuration of Swagger UI.
 * @author jkovac
 */
@Configuration
@ComponentScan("eu.profinit.manta.bachelor.thesis.log.viewer.logic.controllers")
@EnableSwagger2
public class SwaggerConfig {

    /**
     * API version.
     */
    private static final String API_VERSION = "v1";

    /**
     * Swagger
     * @return Swagger docket
     */
    @Bean
    public Docket api() {
        return new Docket(DocumentationType.SWAGGER_2)
                .select()
                .apis(RequestHandlerSelectors.any())
                .paths(regex("/private/log-viewer/v1/.*"))
                .build()
                .apiInfo(apiInfo())
                .tags(
                        new Tag("Log Querying", ""),
                        new Tag("Error expansion", ""),
                        new Tag("Packaging", ""),
                        new Tag("Pruning", ""),
                        new Tag("Settings", ""),
                        new Tag("Full Text Search", ""),
                        new Tag("Log Context", ""),
                        new Tag("Metadata", "")
                )
                .protocols(new HashSet<>(Arrays.asList("http")))
                .genericModelSubstitutes(Optional.class)
                ;
    }

    /**
     * @return Basic information - API documentation header.
     */
    private ApiInfo apiInfo() {
        return new ApiInfo(
                "MANTA Log Viewer API - Bachelor thesis",
                "MANTA Log Viewer API is an HTTP-based interface allowing consumers " +
                        "to view and query logs generated by MANTA workflow executions. The API further allows consumers to" +
                        " create dumps of logs, as well as request to prune specified logs from the repository.",
                API_VERSION,
                null, null, null, null, Collections.emptyList());
    }


}
