package eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * DTO holding relevant data regarding an error for the user
 *
 * @author jkovac
 */
public class ErrorDTO {

    /**
     * Name of the application from which the logs was emitted
     */
    private String application;

    /**
     * Name of the error category
     */
    private String errorCategory;

    /**
     * Name of the error type
     */
    private String errorType;

    /**
     * Impact on the data lineage
     */
    private String impact;

    /**
     * Severity of the error
     */
    private String logLevel;

    /**
     * Solution to the error
     */
    private String solution;

    /**
     * Technical error message
     */
    private String technicalMessage;

    /**
     * User error message
     */
    private String userMessage;

    /**
     * DTO holding relevant data regarding an error for the user
     * @param application Name of the application from which the logs was emitted
     * @param errorCategory Name of the error category
     * @param errorType Name of the error type
     * @param impact Impact on the data lineage
     * @param logLevel Severity of the error
     * @param solution Solution to the error
     * @param technicalMessage Technical error message
     * @param userMessage User error message
     */
    @JsonCreator
    public ErrorDTO(@JsonProperty("application") String application,
                    @JsonProperty("errorCategory") String errorCategory,
                    @JsonProperty("errorType") String errorType,
                    @JsonProperty("impact") String impact,
                    @JsonProperty("logLevel") String logLevel,
                    @JsonProperty("solution") String solution,
                    @JsonProperty("technicalMessage") String technicalMessage,
                    @JsonProperty("userMessage") String userMessage) {
        this.application = application;
        this.errorCategory = errorCategory;
        this.errorType = errorType;
        this.impact = impact;
        this.logLevel = logLevel;
        this.solution = solution;
        this.technicalMessage = technicalMessage;
        this.userMessage = userMessage;
    }


    /**
     * @return Name of the application from which the logs was emitted
     */
    public String getApplication() {
        return application;
    }

    /**
     * @return Name of the error category
     */
    public String getErrorCategory() {
        return errorCategory;
    }

    /**
     * @return Name of the error type
     */
    public String getErrorType() {
        return errorType;
    }

    /**
     * @return Impact on the data lineage
     */
    public String getImpact() {
        return impact;
    }

    /**
     * @return Severity of the error
     */
    public String getLogLevel() {
        return logLevel;
    }

    /**
     * @return Solution to the error
     */
    public String getSolution() {
        return solution;
    }

    /**
     * @return Technical error message
     */
    public String getTechnicalMessage() {
        return technicalMessage;
    }

    /**
     * @return User error message
     */
    public String getUserMessage() {
        return userMessage;
    }
}
