package eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;

/**
 * Response when checking for potentially deleted logs.
 * @author jkovac
 */
public class SetExpiryCheckResponseDTO {

    /**
     * Logs that don't belong to any workflow that may be potentially deleted.
     */
    private int nonWorkflowLogsPotentiallyPruned;

    /**
     * Amount of workflow executions that may be potentially deleted.
     */
    private List<WorkflowExecution> workflowsPotentiallyPruned;

    /**
     * Amount of workflow logs that may be potentially pruned
     */
    private int workflowLogsPotentiallyPruned;

    /**
     * Ctr
     * @param nonWorkflowLogsPotentiallyPruned Logs that don't belong to any workflow that may be potentially deleted
     * @param workflowsPotentiallyPruned Amount of workflow executions that may be potentially deleted
     * @param workflowLogsPotentiallyPruned Amount of workflow logs that may be potentially pruned
     */
    @JsonCreator
    public SetExpiryCheckResponseDTO(@JsonProperty("nonWorkflowLogsPotentiallyPruned") int nonWorkflowLogsPotentiallyPruned,
                                     @JsonProperty("workflowsPotentiallyPruned") List<WorkflowExecution> workflowsPotentiallyPruned,
                                     @JsonProperty("workflowLogsPotentiallyPruned") int workflowLogsPotentiallyPruned) {
        this.nonWorkflowLogsPotentiallyPruned = nonWorkflowLogsPotentiallyPruned;
        this.workflowsPotentiallyPruned = workflowsPotentiallyPruned;
        this.workflowLogsPotentiallyPruned = workflowLogsPotentiallyPruned;
    }

    /**
     * Returns amount of logs that don't belong to any workflow that may be potentially deleted
     * @return amount of potentially pruned non workflow logs
     */
    public int getNonWorkflowLogsPotentiallyPruned() {
        return nonWorkflowLogsPotentiallyPruned;
    }

    /**
     * Returns amount of workflow executions that may be deleted
     * @return list of potentially deleted workflow executions
     */
    public List<WorkflowExecution> getWorkflowsPotentiallyPruned() {
        return workflowsPotentiallyPruned;
    }

    /**
     * Returns amount of workflow logs that may be pruned
     * @return amount of workflow logs potentially pruned
     */
    public int getWorkflowLogsPotentiallyPruned() {
        return workflowLogsPotentiallyPruned;
    }
}
