package eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.executions;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * DTO for storing information about a scenario execution.
 * @author jkovac
 */
public class ScenarioExecutionDTO {

    /**
     * Execution ID of the scenario
     */
    private String scenarioExecutionId;

    /**
     * Technology parameter of the scenario
     */
    private String technology;

    /**
     * Phase to which the scenario belongs
     */
    private String phase;

    /**
     * Qualified name of the scenario
     */
    private String scenarioName;

    /**
     * IDs of connections used during this scenario execution
     */
    private List<String> connections;


    /**
     * Instantiates a new Scenario execution dto.
     *
     * @param scenarioExecutionId Execution ID of the scenario
     * @param technology          Technology parameter of the scenario
     * @param phase               Phase to which the scenario belongs
     * @param scenarioName        Qualified name of the scenario
     * @param connections         Connections used during this scenario execution
     */
    @JsonCreator
    public ScenarioExecutionDTO(@JsonProperty("scenarioExecutionId") String scenarioExecutionId,
                                @JsonProperty("technology") String technology,
                                @JsonProperty("phase") String phase,
                                @JsonProperty("scenarioName") String scenarioName,
                                @JsonProperty("connections") List<String> connections) {
        this.scenarioExecutionId = scenarioExecutionId;
        this.technology = technology;
        this.phase = phase;
        this.scenarioName = scenarioName;
        this.connections = connections;
    }

    /**
     * Gets scenario execution id.
     *
     * @return the scenario execution id
     */
    public String getScenarioExecutionId() {
        return scenarioExecutionId;
    }

    /**
     * Gets technology.
     *
     * @return the technology
     */
    public String getTechnology() {
        return technology;
    }

    /**
     * Gets phase.
     *
     * @return the phase
     */
    public String getPhase() {
        return phase;
    }

    /**
     * Gets scenario name.
     *
     * @return the scenario name
     */
    public String getScenarioName() {
        return scenarioName;
    }

    /**
     * Gets connections.
     *
     * @return the connections
     */
    public List<String> getConnections() {
        return connections;
    }
}
