package eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.executions;

import java.util.Date;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * DTO for transferring stored executions metadata
 * @author jkovac
 */
public class WorkflowExecutionDTO {

    /**
     * ID of the workflow
     */
    private String workflowExecutionId;

    /**
     * Timestamp when workflow started being executed
     */
    private Date timestamp;

    /**
     * List of scenario executions belonging to this workflow execution
     */
    private List<ScenarioExecutionDTO> scenarioExecutions;

    /**
     * Instantiates a new Workflow execution dto.
     *
     * @param workflowExecutionId the workflow execution id
     * @param timestamp           the timestamp
     * @param scenarioExecutions  the scenario executions
     */
    @JsonCreator
    public WorkflowExecutionDTO(@JsonProperty("workflowExecutionId") String workflowExecutionId,
                                @JsonProperty("timestamp") Date timestamp,
                                @JsonProperty("scenarioExecutions") List<ScenarioExecutionDTO> scenarioExecutions) {
        this.workflowExecutionId = workflowExecutionId;
        this.timestamp = timestamp;
        this.scenarioExecutions = scenarioExecutions;
    }

    /**
     * Gets workflow execution id.
     *
     * @return the workflow execution id
     */
    public String getWorkflowExecutionId() {
        return workflowExecutionId;
    }

    /**
     * Gets timestamp.
     *
     * @return the timestamp
     */
    public Date getTimestamp() {
        return timestamp;
    }

    /**
     * Gets scenario executions.
     *
     * @return the scenario executions
     */
    public List<ScenarioExecutionDTO> getScenarioExecutions() {
        return scenarioExecutions;
    }
}
