package eu.profinit.manta.bachelor.thesis.log.viewer.errors.categories;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.BaseCategory;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;

/**
 * Errors occurring during repository manipulation.
 * @author jkovac
 */
public class RepositoryErrors extends BaseCategory {

    /**
     * Error thrown due to missing pruning.properties file in resources.
     * @return error builder
     */
    @Error(
            userMessage = "Pruning setting will not be changed due to error.",
            technicalMessage = "Pruning configuration file has not been found.",
            solution = "Report it.",
            severity = Severity.ERROR
    )
    public PruningConfigurationNotFoundBuilder pruningConfigurationNotFound(){
        return new PruningConfigurationNotFoundBuilder(this);
    }

    /**
     * Error thrown when it is not possible to create a log dump for given scenario execution.
     * @return error builder
     */
    @Error(
            userMessage = "Unable to create dump due to invalid file path.",
            technicalMessage = "Unable to create database dump for %{entity} due to IOException.",
            solution = "Report it.",
            severity = Severity.ERROR
    )
    public UnableToCreateDumpBuilder unableToCreateDump(){
        return new UnableToCreateDumpBuilder(this);
    }

}
