package eu.profinit.manta.bachelor.thesis.log.viewer.model;

import org.apache.ibatis.annotations.Param;

/**
 * POJO representing an error definition
 * @author jkovac
 */
public class ErrorDefinition {

    /**
     * DB id of the error definition
     */
    private Long errorDefinitionId;

    /**
     * Error category
     */
    private String errorCategory;

    /**
     * Error type
     */
    private String errorType;

    /**
     * User error message
     */
    private String userMessage;

    /**
     * Technical error message
     */
    private String technicalMessage;

    /**
     * Solution to the error
     */
    private String solution;

    /**
     * Impact on the lineage
     */
    private String impact;

    /**
     * Severity
     */
    private String logLevel;

    /**
     *  Ctr for the error definition
     * @param errorDefinitionId DB id of the error definition
     * @param errorCategory Error category
     * @param errorType Error type
     * @param userMessage User error message
     * @param technicalMessage Technical error message
     * @param solution Solution to the error
     * @param impact Impact on the lineage
     * @param logLevel Severity
     */
    public ErrorDefinition(@Param("errorDefinitionId") Long errorDefinitionId,
                           @Param("errorCategory") String errorCategory,
                           @Param("errorType") String errorType,
                           @Param("userMessage") String userMessage,
                           @Param("technicalMessage") String technicalMessage,
                           @Param("solution") String solution,
                           @Param("impact") String impact,
                           @Param("logLevel") String logLevel) {
        this.errorDefinitionId = errorDefinitionId;
        this.errorCategory = errorCategory;
        this.errorType = errorType;
        this.userMessage = userMessage;
        this.technicalMessage = technicalMessage;
        this.solution = solution;
        this.impact = impact;
        this.logLevel = logLevel;
    }

    /**
     *  Ctr for the error definition
     * @param errorCategory Error category
     * @param errorType Error type
     * @param userMessage User error message
     * @param technicalMessage Technical error message
     * @param solution Solution to the error
     * @param impact Impact on the lineage
     * @param logLevel Severity
     */
    public ErrorDefinition(String errorCategory,
                           String errorType,
                           String userMessage,
                           String technicalMessage,
                           String solution,
                           String impact,
                           String logLevel) {
        this.errorCategory = errorCategory;
        this.errorType = errorType;
        this.userMessage = userMessage;
        this.technicalMessage = technicalMessage;
        this.solution = solution;
        this.impact = impact;
        this.logLevel = logLevel;
    }

    /**
     * Gets error definition id.
     *
     * @return the error definition id
     */
    public Long getErrorDefinitionId() {
        return errorDefinitionId;
    }

    /**
     * Sets error definition id.
     *
     * @param errorDefinitionId the error definition id
     */
    public void setErrorDefinitionId(Long errorDefinitionId) {
        this.errorDefinitionId = errorDefinitionId;
    }

    /**
     * Gets error category.
     *
     * @return the error category
     */
    public String getErrorCategory() {
        return errorCategory;
    }

    /**
     * Gets error type.
     *
     * @return the error type
     */
    public String getErrorType() {
        return errorType;
    }

    /**
     * Gets user message.
     *
     * @return the user message
     */
    public String getUserMessage() {
        return userMessage;
    }

    /**
     * Gets technical message.
     *
     * @return the technical message
     */
    public String getTechnicalMessage() {
        return technicalMessage;
    }

    /**
     * Gets solution.
     *
     * @return the solution
     */
    public String getSolution() {
        return solution;
    }

    /**
     * Gets impact.
     *
     * @return the impact
     */
    public String getImpact() {
        return impact;
    }

    /**
     * Gets log level.
     *
     * @return the log level
     */
    public String getLogLevel() {
        return logLevel;
    }
}
