package eu.profinit.manta.bachelor.thesis.log.viewer.model;

import org.apache.ibatis.annotations.Param;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * POJO representing the created/uploaded package metadata
 * @author jkovac
 */
public class PackageMetadata {

    /**
     * Db id of the connection from which the package is created
     */
    @JsonIgnore
    private Long connDbId;

    /**
     * Name of the connection
     */
    private String connectionId;

    /**
     * Name of the scenario
     */
    private String scenarioName;

    /**
     * Name of the scenario phase
     */
    private String phase;

    /**
     * Name of the technology
     */
    private String technology;

    /**
     * Ctr of the package metadata
     * @param connectionId Name of the connection
     * @param scenarioName Name of the scenario
     * @param phase Name of the scenario phase
     * @param technology Name of the technology
     */
    @JsonCreator
    public PackageMetadata(@JsonProperty("connectionId") String connectionId,
                           @JsonProperty("scenarioName") String scenarioName,
                           @JsonProperty("phase") String phase,
                           @JsonProperty("technology") String technology) {
        this.connectionId = connectionId;
        this.scenarioName = scenarioName;
        this.phase = phase;
        this.technology = technology;
    }

    /**
     * Ctr of the package metadata
     * @param connectionId Name of the connection
     * @param scenarioName Name of the scenario
     * @param phase Name of the scenario phase
     * @param technology Name of the technology
     * @param connDbId db if of the connection
     */
    public PackageMetadata(@Param("connDbId") Long connDbId,
                           @Param("connectionId") String connectionId,
                           @Param("scenarioName") String scenarioName,
                           @Param("phase") String phase,
                           @Param("technology") String technology) {
        this.connDbId = connDbId;
        this.connectionId = connectionId;
        this.scenarioName = scenarioName;
        this.phase = phase;
        this.technology = technology;
    }

    /**
     * Gets conn db id.
     *
     * @return the conn db id
     */
    public Long getConnDbId() {
        return connDbId;
    }

    /**
     * Gets connection id.
     *
     * @return the connection id
     */
    public String getConnectionId() {
        return connectionId;
    }

    /**
     * Gets scenario name.
     *
     * @return the scenario name
     */
    public String getScenarioName() {
        return scenarioName;
    }

    /**
     * Gets phase.
     *
     * @return the phase
     */
    public String getPhase() {
        return phase;
    }

    /**
     * Gets technology.
     *
     * @return the technology
     */
    public String getTechnology() {
        return technology;
    }
}
