package eu.profinit.manta.bachelor.thesis.log.viewer.model;

import org.apache.ibatis.annotations.Param;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Model representing one uploaded package
 * @author jkovac
 */
public class UploadedPackage {

    /**
     * Name of the package
     */
    private String packageName;

    /**
     * Technology parameter of the scenario
     */
    private String technology;

    /**
     * Phase to which the scenario belongs
     */
    private String phase;

    /**
     * Qualified name of the scenario
     */
    private String scenarioName;

    /**
     * Name of the connection
     */
    private String connection;

    /**
     * Name of the application
     */
    private String application;

    /**
     * Ctr
     * @param packageName Name of the package
     * @param technology Technology parameter of the scenario
     * @param phase Phase to which the scenario belongs
     * @param scenarioName Qualified name of the scenario
     * @param connection connection name
     */
    public UploadedPackage(String packageName,
                           String technology,
                           String phase,
                           String scenarioName,
                           String connection) {
        this.packageName = packageName;
        this.technology = technology;
        this.phase = phase;
        this.scenarioName = scenarioName;
        this.connection = connection;
        this.application = "";
    }

    /**
     * Ctr
     * @param packageName Name of the package
     * @param technology Technology parameter of the scenario
     * @param phase Phase to which the scenario belongs
     * @param scenarioName Qualified name of the scenario
     * @param application name of the application
     * @param connection name of the connection
     */
    @JsonCreator
    public UploadedPackage(@JsonProperty("packageName") @Param("packageName") String packageName,
                           @JsonProperty("technology") @Param("technology") String technology,
                           @JsonProperty("phase") @Param("phase") String phase,
                           @JsonProperty("scenarioName") @Param("scenarioName") String scenarioName,
                           @JsonProperty("connection") @Param("connection") String connection,
                           @JsonProperty("application") @Param("application") String application) {
        this.packageName = packageName;
        this.technology = technology;
        this.phase = phase;
        this.scenarioName = scenarioName;
        this.application = application;
        this.connection = connection;
    }

    /**
     * Ctr
     * @param packageName Name of the package
     * @param application application from which logs were packaged
     */
    public UploadedPackage(String packageName, String application) {
        this.packageName = packageName;
        this.application = application;
        this.technology = "";
        this.phase = "";
        this.scenarioName = "";
        this.connection = "";
    }

    /**
     * Name of the package
     * @return package name
     */
    public String getPackageName() {
        return packageName;
    }

    /**
     * Technology parameter of the scenario
     * @return technology name
     */
    public String getTechnology() {
        return technology;
    }

    /**
     * Phase to which the scenario belongs
     * @return scenario name
     */
    public String getPhase() {
        return phase;
    }

    /**
     * Qualified name of the scenario
     * @return scenario name
     */
    public String getScenarioName() {
        return scenarioName;
    }

    /**
     * Gets application.
     *
     * @return the application
     */
    public String getApplication() {
        return application;
    }

    /**
     * Gets connection.
     *
     * @return the connection
     */
    public String getConnection() {
        return connection;
    }

    /**
     * Check if two packages are equal, used by tests
     * @param other other package
     * @return boolean same or not
     */
    public boolean equals(Object other){
        UploadedPackage otherPackage = (UploadedPackage) other;
        return this.packageName.equals(otherPackage.getPackageName())
                &&
                this.connection.equals(otherPackage.getConnection())
                &&
                this.application.equals(otherPackage.getApplication())
                &&
                this.phase.equals(otherPackage.getPhase())
                &&
                this.scenarioName.equals(otherPackage.getScenarioName())
                &&
                this.technology.equals(otherPackage.getTechnology())
                ;
    }
}
