package eu.profinit.manta.bachelor.thesis.log.viewer.model;

import java.time.Instant;
import java.util.Date;

import org.apache.ibatis.annotations.Param;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * POJO representing one execution of a workflow, i.e. one run
 * @author jkovac
 */
public class WorkflowExecution {

    /**
     * ID of the workflow
     */
    private String workflowExecutionId;

    /**
     * Timestamp when workflow started being executed
     */
    private Date timestamp;

    /**
     * POJO representing one execution of a workflow, i.e. one run
     * @param workflowExecutionId id of the workflow
     */
    public WorkflowExecution(String workflowExecutionId) {
        this.workflowExecutionId = workflowExecutionId;
        this.timestamp = Date.from(Instant.now());
    }

    /**
     * Constructor for mybatis mapping
     * @param workflowExecutionId id of the workflow
     * @param timestamp timestamp of the start of the workflow
     */
    @JsonCreator
    public WorkflowExecution(@Param("workflowExecutionId") @JsonProperty("workflowExecutionId") String workflowExecutionId,
                             @Param("timestamp") @JsonProperty("timestamp") Date timestamp) {
        this.workflowExecutionId = workflowExecutionId;
        this.timestamp = new Date(timestamp.getTime());
    }

    /**
     * @return id of the workflow
     */
    public String getWorkflowExecutionId() {
        return workflowExecutionId;
    }

    /**
     * @return execution start time
     */
    public Date getTimestamp() {
        return new Date(timestamp.getTime());
    }

    /**
     * Check if two workflow executions are same, used by tests
     * @param other other workflow execution
     * @return boolean whether they are same
     */
    @Override
    public boolean equals(Object other){
        WorkflowExecution otherExecution = (WorkflowExecution) other;
        return this.workflowExecutionId.equals(otherExecution.getWorkflowExecutionId()) && this.timestamp.equals(otherExecution.getTimestamp());
    }
}
