package eu.profinit.manta.bachelor.thesis.log.viewer.model.logs;

import org.apache.ibatis.annotations.Param;

/**
 * Metadata of the log used for retrieving its context
 * @author jkovac
 */
public class LogMetadata {

    /**
     * Type of the log
     */
    private String type;

    /**
     * DB id of the connection
     */
    private Long connDbId;

    /**
     * Workflow execution id
     */
    private String workflowExecutionId;

    /**
     * Scenario execution id
     */
    private String scenarioExecutionId;

    /**
     * Name of the package
     */
    private String packageName;

    /**
     * Ctr for the log metadata
     * @param type Type of the log
     * @param connDbId DB id of the connection
     * @param workflowExecutionId Workflow execution id
     * @param scenarioExecutionId Scenario execution id
     * @param packageName Name of the package
     */
    public LogMetadata(@Param("type") String type,
                       @Param("connDbId") Long connDbId,
                       @Param("workflowExecutionId") String workflowExecutionId,
                       @Param("scenarioExecutionId") String scenarioExecutionId,
                       @Param("packageName") String packageName) {
        this.connDbId = connDbId;
        this.workflowExecutionId = workflowExecutionId;
        this.scenarioExecutionId = scenarioExecutionId;
        this.packageName = packageName;
        this.type = type;
    }

    /**
     * Gets type.
     *
     * @return the type
     */
    public String getType() {
        return type;
    }

    /**
     * Gets conn db id.
     *
     * @return the conn db id
     */
    public Long getConnDbId() {
        return connDbId;
    }

    /**
     * Gets workflow execution id.
     *
     * @return the workflow execution id
     */
    public String getWorkflowExecutionId() {
        return workflowExecutionId;
    }

    /**
     * Gets scenario execution id.
     *
     * @return the scenario execution id
     */
    public String getScenarioExecutionId() {
        return scenarioExecutionId;
    }

    /**
     * Gets package name.
     *
     * @return the package name
     */
    public String getPackageName() {
        return packageName;
    }
}
