package eu.profinit.manta.bachelor.thesis.log.viewer.model.logs;

import org.apache.ibatis.annotations.Param;

/**
 * POJO representing a log read from a CSV dump and to be uploaded as a package log
 * @author jkovac
 */
public class UploadedLog {

    /**
     * ERROR or DEBUG
     */
    private String type;

    /**
     * Application that emitted the log
     */
    private String application;

    /**
     * context of the log
     */
    private String context;

    /**
     * Name of the thread
     */
    private String thread;

    /**
     * ID of the thread
     */
    private int threadId;

    /**
     * Name of the logger
     */
    private String loggerName;

    /**
     * Worker number
     */
    private int workerNumber;

    /**
     * Level of the log, DEBUG, INFO...
     */
    private String logLevel;

    /**
     * Epoch second of the log
     */
    private long epochSecond;

    /**
     * Nano second of the log
     */
    private long nanoOfSecond;

    /**
     * Log message, empty for ERROR type
     */
    private String message;

    /**
     * Name of the exception
     */
    private String exceptionName;

    /**
     * Stacktrace
     */
    private String extendedStacktrace;

    /**
     * DB id of the error definition
     */
    private Long errorDefinitionId;

    /**
     * Instantiates a new Uploaded log.
     *
     * @param type               the type
     * @param application        Application that emitted the log
     * @param context            context of the log
     * @param thread             the thread
     * @param threadId           the thread id
     * @param loggerName         the logger name
     * @param workerNumber       the worker number
     * @param logLevel           the log level
     * @param epochSecond        the epoch second
     * @param nanoOfSecond       the nano of second
     * @param message            the message
     * @param exceptionName      the exception name
     * @param extendedStacktrace the extended stacktrace
     * @param errorDefinitionId  the error definition id
     */
    public UploadedLog(@Param("type") String type,
                       @Param("application") String application,
                       @Param("context") String context,
                       @Param("thread") String thread,
                       @Param("threadId") int threadId,
                       @Param("loggerName") String loggerName,
                       @Param("workerNumber") int workerNumber,
                       @Param("logLevel") String logLevel,
                       @Param("epochSecond") long epochSecond,
                       @Param("nanoOfSecond") long nanoOfSecond,
                       @Param("message") String message,
                       @Param("exceptionName") String exceptionName,
                       @Param("extendedStacktrace") String extendedStacktrace,
                       @Param("errorDefinitionId") Long errorDefinitionId) {
        this.type = type;
        this.application = application;
        this.context = context;
        this.thread = thread;
        this.threadId = threadId;
        this.loggerName = loggerName;
        this.workerNumber = workerNumber;
        this.logLevel = logLevel;
        this.epochSecond = epochSecond;
        this.nanoOfSecond = nanoOfSecond;
        this.message = message;
        this.exceptionName = exceptionName;
        this.extendedStacktrace = extendedStacktrace;
        this.errorDefinitionId = errorDefinitionId;
    }

    /**
     * Gets type.
     *
     * @return the type
     */
    public String getType() {
        return type;
    }

    /**
     * Gets application.
     *
     * @return the application
     */
    public String getApplication() {
        return application;
    }

    /**
     * Gets context.
     *
     * @return the context
     */
    public String getContext() {
        return context;
    }

    /**
     * Gets thread.
     *
     * @return the thread
     */
    public String getThread() {
        return thread;
    }

    /**
     * Gets thread id.
     *
     * @return the thread id
     */
    public int getThreadId() {
        return threadId;
    }

    /**
     * Gets logger name.
     *
     * @return the logger name
     */
    public String getLoggerName() {
        return loggerName;
    }

    /**
     * Gets worker number.
     *
     * @return the worker number
     */
    public int getWorkerNumber() {
        return workerNumber;
    }

    /**
     * Gets log level.
     *
     * @return the log level
     */
    public String getLogLevel() {
        return logLevel;
    }

    /**
     * Gets epoch second.
     *
     * @return the epoch second
     */
    public long getEpochSecond() {
        return epochSecond;
    }

    /**
     * Gets nano of second.
     *
     * @return the nano of second
     */
    public long getNanoOfSecond() {
        return nanoOfSecond;
    }

    /**
     * Gets message.
     *
     * @return the message
     */
    public String getMessage() {
        return message;
    }

    /**
     * Gets exception name.
     *
     * @return the exception name
     */
    public String getExceptionName() {
        return exceptionName;
    }

    /**
     * Gets extended stacktrace.
     *
     * @return the extended stacktrace
     */
    public String getExtendedStacktrace() {
        return extendedStacktrace;
    }

    /**
     * Gets error definition id.
     *
     * @return the error definition id
     */
    public Long getErrorDefinitionId() {
        return errorDefinitionId;
    }
}
