package eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.additionalcontext;

import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Class holding data from ThreadContext common for all logs.
 * @author jkovac
 */
public class BaseContextMap {

    /**
     * Technology provided from the LoggingContext
     */
    private String technology;

    /**
     * Phase of the scenario provided from the LoggingContext
     */
    private String phase;

    /**
     * Name of the scenario
     */
    private String scenario;

    /**
     * Workflow (run) ID of which the scenario is a part of
     */
    private String workflowExecutionId;

    /**
     * Execution ID of a scenario to which the scenario belongs
     */
    private String scenarioExecutionId;

    /**
     * ConnectionId provided from the LoggingContext
     */
    private String connectionId;

    /**
     * Context node provided by LoggingContext
     */
    private String context;

    /**
     * Worker number
     */
    private int workerNumber;

    /**
     * Name of the package to which the log belongs
     */
    private String packageName;

    /**
     * Default constructor.
     */
    public BaseContextMap(){
    }

    /**
     * Holder of data from ThreadContext common for all logs.
     * @param workflowExecutionId workflow (run) id
     * @param scenarioExecutionId execution id of a scenario to which the log belongs
     * @param connectionId ConnectionId provided from the LoggingContext
     * @param context Context node provided by LoggingContext
     * @param workerNumber worker number
     * @param scenario scenario name
     * @param phase phase of the scenario
     * @param technology technology of the scenario
     */
    public BaseContextMap(@JsonProperty("WORKFLOW_ID") String workflowExecutionId,
                          @JsonProperty("EXECUTION_ID") String scenarioExecutionId,
                          @JsonProperty("TECHNOLOGY") String technology,
                          @JsonProperty("PHASE") String phase,
                          @JsonProperty("SCENARIO") String scenario,
                          @JsonProperty("CONNECTION_ID") String connectionId,
                          @JsonProperty("CONTEXT") String context,
                          @JsonProperty("workerNumber") int workerNumber) {
        this.workflowExecutionId = workflowExecutionId;
        this.scenarioExecutionId = scenarioExecutionId;
        this.connectionId = connectionId;
        this.context = context;
        this.workerNumber = workerNumber;
        this.technology = technology;
        this.phase = phase;
        this.scenario = scenario;
    }

    /**
     * Holder of data from ThreadContext common for all logs.
     * @param workflowExecutionId workflow (run) id
     * @param scenarioExecutionId execution id of a scenario to which the log belongs
     * @param technology technology of the scenario
     * @param phase phase to which the scenario belongs
     * @param packageName name of the package to which the log belongs
     * @param scenario name of the scenario
     * @param connectionId ConnectionId provided from the LoggingContext
     * @param context Context node provided by LoggingContext
     * @param workerNumber worker number
     */
    public BaseContextMap(String workflowExecutionId,
                          String scenarioExecutionId,
                          String technology,
                          String phase,
                          String scenario,
                          String connectionId,
                          String context,
                          int workerNumber,
                          String packageName) {
        this.workflowExecutionId = workflowExecutionId;
        this.scenarioExecutionId = scenarioExecutionId;
        this.connectionId = connectionId;
        this.context = context;
        this.workerNumber = workerNumber;
        this.technology = technology;
        this.phase = phase;
        this.scenario = scenario;
        this.packageName = packageName;
    }

    /**
     * @return Workflow (run) ID of which the scenario is a part of
     */
    public String getWorkflowExecutionId() {
        return workflowExecutionId;
    }

    /**
     * @return Execution ID of a scenario to which the log belongs
     */
    public String getScenarioExecutionId() {
        return scenarioExecutionId;
    }

    /**
     * @return Context node provided by LoggingContext
     */
    public String getContext() {
        return context;
    }

    /**
     * @return Worker number
     */
    public int getWorkerNumber() {
        return workerNumber;
    }

    /**
     * @return ConnectionId provided from the LoggingContext
     */
    public String getConnectionId() {
        return connectionId;
    }

    /**
     * @return Technology provided from the LoggingContext
     */
    public String getTechnology() {
        return technology;
    }

    /**
     * @return Phase of the scenario provided from the LoggingContext
     */
    public String getPhase() {
        return phase;
    }

    /**
     * @return Name of the scenario
     */
    public String getScenario() {
        return scenario;
    }

    /**
     * Gets package name.
     *
     * @return the package name
     */
    public String getPackageName() {
        return packageName;
    }
}
