package eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.additionalcontext;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Class holding data from ThreadContext belonging to error logs.
 * @author jkovac
 */
public class ErrorContextMap extends BaseContextMap {

    /**
     * User-friendly error message
     */
    private String userMessage;

    /**
     * Message with technical details
     */
    private String technicalMessage;

    /**
     * Solution for the error
     */
    private String solution;

    /**
     * Impact of the error on the data lineage
     */
    private String lineageImpact;

    /**
     * Category of the error
     */
    private String category;

    /**
     * Name of the error
     */
    private String errorType;

    /**
     * Default constructor.
     */
    public ErrorContextMap(){
    }

    /**
     * Holder of data from ThreadContext for error logs.
     * @param workflowExecutionId workflow (run) id
     * @param scenarioExecutionId execution id of a scenario to which the log belongs
     * @param technology Technology provided from the LoggingContext
     * @param connectionId ConnectionId provided from the LoggingContext
     * @param phase Phase of the scenario provided from the LoggingContext
     * @param scenario Name of the scenario
     * @param context Context node provided by LoggingContext
     * @param userMessage User-friendly error message
     * @param technicalMessage Message with technical details
     * @param solution Solution for the error
     * @param lineageImpact Impact of the error on the data lineage
     * @param category Category of the error
     * @param errorType Name of the error
     * @param workerNumber worker number
     */
    @JsonCreator
    public ErrorContextMap(@JsonProperty("WORKFLOW_ID") String workflowExecutionId,
                           @JsonProperty("EXECUTION_ID") String scenarioExecutionId,
                           @JsonProperty("TECHNOLOGY") String technology,
                           @JsonProperty("CONNECTION_ID") String connectionId,
                           @JsonProperty("PHASE") String phase,
                           @JsonProperty("SCENARIO") String scenario,
                           @JsonProperty("CONTEXT") String context,
                           @JsonProperty("workerNumber") int workerNumber,
                           @JsonProperty("USER_MESSAGE") String userMessage,
                           @JsonProperty("TECHNICAL_MESSAGE") String technicalMessage,
                           @JsonProperty("SOLUTION") String solution,
                           @JsonProperty("IMPACT") String lineageImpact,
                           @JsonProperty("CATEGORY") String category,
                           @JsonProperty("ERROR_TYPE") String errorType) {
        super(workflowExecutionId, scenarioExecutionId, technology, phase, scenario, connectionId, context, workerNumber);

        this.userMessage = userMessage;
        this.technicalMessage = technicalMessage;
        this.solution = solution;
        this.lineageImpact = lineageImpact;
        this.category = category;
        this.errorType = errorType;
    }

    /**
     * Holder of data from ThreadContext for error logs.
     * @param workflowExecutionId workflow (run) id
     * @param scenarioExecutionId execution id of a scenario to which the log belongs
     * @param technology Technology provided from the LoggingContext
     * @param connectionId ConnectionId provided from the LoggingContext
     * @param phase Phase of the scenario provided from the LoggingContext
     * @param scenario Name of the scenario
     * @param context Context node provided by LoggingContext
     * @param userMessage User-friendly error message
     * @param technicalMessage Message with technical details
     * @param solution Solution for the error
     * @param lineageImpact Impact of the error on the data lineage
     * @param category Category of the error
     * @param errorType Name of the error
     * @param workerNumber worker number
     * @param packageName name of the package to which the error log belongs
     */
    public ErrorContextMap(String workflowExecutionId,
                           String scenarioExecutionId,
                           String technology,
                           String connectionId,
                           String phase,
                           String scenario,
                           String context,
                           int workerNumber,
                           String userMessage,
                           String technicalMessage,
                           String solution,
                           String lineageImpact,
                           String category,
                           String errorType,
                           String packageName) {
        super(workflowExecutionId, scenarioExecutionId, technology, phase, scenario, connectionId, context, workerNumber, packageName);

        this.userMessage = userMessage;
        this.technicalMessage = technicalMessage;
        this.solution = solution;
        this.lineageImpact = lineageImpact;
        this.category = category;
        this.errorType = errorType;
    }

    /**
     * @return User-friendly error message
     */
    public String getUserMessage() {
        return userMessage;
    }

    /**
     * @return Message with technical details
     */
    public String getTechnicalMessage() {
        return technicalMessage;
    }

    /**
     * @return Solution for the error
     */
    public String getSolution() {
        return solution;
    }

    /**
     * @return Impact of the error on the data lineage
     */
    public String getLineageImpact() {
        return lineageImpact;
    }

    /**
     * @return Category of the error
     */
    public String getCategory() {
        return category;
    }

    /**
     * @return Name of the error
     */
    public String getErrorType() {
        return errorType;
    }
}
