package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;

/**
 * DAO for the Connections table
 * @author jkovac
 */
public interface ConnectionsDAO {

    /**
     * Retrieve a connection by its database ID
     * @param connDbId database id
     * @return Connection that is stored under it
     */
    Connection find(Long connDbId);

    /**
     * Get database id for a record that has same properties
     * @param connection wanted connection
     * @return DB id
     */
    Long getIdByData(Connection connection);

    /**
     * Get all connections stored in the repository
     * @return list of all connections
     */
    List<Connection> getAll();

    /**
     * Insert a new connection used in a scenario execution into a database
     * @param connection connection to be inserted
     * @return amount of inserted rows
     */
    Long save(Connection connection);

    /**
     * Delete a connection from the repository
     * @param connection connection to be deleted
     * @return amount of deleted rows
     */
    Long delete(Connection connection);
}
