package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;

/**
 * DAO for error_definitions table and operations concerning it
 * @author jkovac
 */
public interface ErrorDefinitionDAO {

    /**
     * Get DB id by data
     * @param errorDefinition wanted definition
     * @return ID
     */
    Long getIdByData(ErrorDefinition errorDefinition);

    /**
     * Persist a new error definition
     * @param errorDefinition definition to be inserted
     * @return amount of inserted rows
     */
    Long save(ErrorDefinition errorDefinition);

    /**
     * Get definition by ID
     * @param errorDefinitionId DB ID
     * @return error definition stored under the given ID
     */
    ErrorDefinition find(Long errorDefinitionId);

    /**
     * Get distinct ids that are currently used by the error metadata table
     * @return List of IDs of distinct definitions
     */
    List<Long> getDistinct();

    /**
     * Get all error definitions
     * @return List of ids of all stored definitions
     */
    List<Long> getAll();

    /**
     * Delete all definitions stored under given ids
     * @param errorDefinitionIds list of ids to delete
     * @return amount of deleted rows
     */
    Long delete(List<Long> errorDefinitionIds);
}
