package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;

/**
 * DAO for error metadata table and operations concerning it
 * @author jkovac
 */
public interface ErrorMetadataDAO {

    /**
     * Find metadata by id
     * @param id id of the wanted metadata
     * @return metadata stored under the given id
     */
    ErrorMetadata find(Long id);

    /**
     * Get id of the given metadata
     * @param errorMetadata wanted metadata
     * @return id of the metadata
     */
    Long getIdByData(ErrorMetadata errorMetadata);

    /**
     * Insert a new error metadata object to repository
     * @param errorMetadata error metadata
     * @return amount of rows inserted
     */
    Long save(ErrorMetadata errorMetadata);

    /**
     * @return List of all metadata IDs
     */
    List<Long> getAllIds();

    /**
     * Get list of distinct error metadata currently used in the logs table, used for automated pruning
     * @return list of distinct error metadata ids in logs
     */
    List<Long> getDistinctIds();

    /**
     * Delete given error metadata
     * @param errorIds ids to delete
     * @return amount of rows deleted
     */
    Long delete(List<Long> errorIds);
}
