package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.PackageMetadata;

/**
 * DAO for package creation queries
 * @author jkovac
 */
public interface PackageCreateDAO {

    /**
     * Create CSV dump of error definitions that belong to connection of which is a package being created
     * @param connDbId db id of the connection
     * @param path path where to create the dump
     */
    void createErrorDefinitionsDumpForConnection(Long connDbId, String path);

    /**
     * Create CSV dump of error definitions that belong to application of which is a package being created
     * @param application name of the application that is being packaged
     * @param path path to the dump
     */
    void createErrorDefinitionsDumpForApplication(String application, String path);

    /**
     * Creates CSV dump of logs for a given application.
     * @param application application of which logs should be exported
     * @param path file path to the dump
     */
    void createLogDumpForApplication(String application, String path);

    /**
     * Creates CSV dump of logs.
     * @param path path to the dump
     * @param connDbId connection db id
     */
    void createLogDumpForConnection(Long connDbId, String path);

    /**
     * Get package metadata for a given connection (technology, phase, scenario name)
     * @param connection connection that is being packaged
     * @return package metadata
     */
    PackageMetadata getPackageMetadata(Connection connection);

}
