package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;

/**
 * DAO for uploaded packages
 * @author jkovac
 */
public interface PackageUploadDAO {
    /**
     * Insert metadata about a new uploaded package to the DB
     * @param uploadedPackage metadata about the package
     * @return amount of inserted rows
     */
    Long save(UploadedPackage uploadedPackage);

    /**
     * Read CSV error definition dump
     * @param absolutePath path to the dump
     * @return read error definitions
     */
    List<ErrorDefinition> readErrorDefinitions(String absolutePath);

    /**
     * Read CSV log dump
     * @param path path to the dump
     * @return read logs
     */
    List<UploadedLog> readLogs(String path);

    /**
     * Insert a new error log belonging to a package
     * @param log log to be inserted
     * @param errorId error metadata id
     * @param packageName name of the uploaded package
     * @return amount of inserted rows
     */
    Long upload(UploadedLog log, Long errorId, String packageName);

    /**
     * Insert a new debug log belonging to a package
     * @param log log to be inserted
     * @param packageName package name
     * @return amount of inserted rows
     */
    Long upload(UploadedLog log, String packageName);
}
