package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;

/**
 * DAO for queries regarding possible consequences of changing prune settings
 * @author jkovac
 */
public interface PruneCheckingDAO {

    /**
     * Get workflow executions that would be deleted if settings were changed to a given value
     * @param pruneAfter new settings
     * @return workflow executions potentially deleted
     */
    List<WorkflowExecution> getPotentiallyDeletedWorkflows(double pruneAfter);

    /**
     * Get amount of logs that would be deleted that don't belong to a workflow execution
     * @param pruneAfter new settings
     * @return amount of non-workflow logs deleted
     */
    Integer getAmountOfPotentiallyDeletedLogs(double pruneAfter);

    /**
     * Get amount of logs that would be deleted that belong to passed workflow executions
     * @param pruneAfter new settings
     * @param deletedWorkflowExecutions potentially deleted workflow executions
     * @return amount of logs deleted
     */
    Integer getAmountOfPotentiallyDeletedWorkflowLogs(double pruneAfter, List<WorkflowExecution> deletedWorkflowExecutions);

}
