package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.DistinctErrorType;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;

/**
 * DAO for retrieving metadata about the repository
 * @author jkovac
 */
public interface RepositoryMetadataDAO {

    /**
     * @return Distinct applications stored in the repository
     */
    List<String> getDistinctApplications();

    /**
     * Distinct error types fitting a passed filter
     * @param filter request filter
     * @return distinct error types
     */
    List<DistinctErrorType> getDistinctErrorTypes(Filter filter);

    /**
     * Distinct categories stored in the repo
     * @param filter request filter
     * @return distinct categories
     */
    List<String> getDistinctCategories(Filter filter);

    /**
     * Get connections that were used in a passed scenario execution
     * @param scenarioExecutionId scenario execution id
     * @return connection IDs used in the scenario execution
     */
    List<String> getConnectionsForScenario(String scenarioExecutionId);
}
