package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;
import java.util.Optional;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;

/**
 * DAO for accessing scenario executions table
 * @author jkovac
 */
public interface ScenarioExecutionDAO {
    /**
     * Finds execution by given execution ID.
     * @param scenarioExecutionId execution ID of the queried execution
     * @return Query result
     */
    Optional<ScenarioExecution> find(String scenarioExecutionId);

    /**
     * Persists a new scenario execution
     * @param scenarioExecution scenario execution that should be persisted
     * @return amount of inserted rows
     */
    Long save(ScenarioExecution scenarioExecution);

    /**
     * Lists all scenario executions for a given workflow execution ID.
     * @param workflowExecutionId ID of the workflow execution
     * @return list of the found scenario executions
     */
    List<ScenarioExecution> getAllFor(String workflowExecutionId);

    /**
     * Delete a scenario execution
     * @param scenarioExecutionId id of the scenario execution
     * @return amount of deleted rows
     */
    Long delete(String scenarioExecutionId);
}
