package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;
import java.util.Optional;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;

/**
 * DAO for accessing workflow executions table
 * @author jkovac
 */
public interface WorkflowExecutionDAO {

    /**
     * Queries a workflow by a given ID.
     * @param workflowExecutionId id of the queried workflow
     * @return queried workflow
     */
    Optional<WorkflowExecution> find(String workflowExecutionId);

    /**
     * Insert a new workflow execution to the repository
     * @param workflowExecution workflow execution to be persisted
     * @return amount of inserted rows
     */
    Long save(WorkflowExecution workflowExecution);

    /**
     * Returns all persisted workflow executions
     * @return persisted workflow executions
     */
    List<WorkflowExecution> getAll();

    /**
     * Deletes a workflow execution. Delete cascades to scenario executions table and logs table.
     * @param workflowExecution workflow execution that should be deleted
     * @return amount of inserted rows
     */
    Long delete(WorkflowExecution workflowExecution);


}
