package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.ErrorDefinitionsMapper;

/**
 * Error Definition DAO implementation
 * @author jkovac
 */
@Repository
public class ErrorDefinitionDAOImpl implements ErrorDefinitionDAO {

    /**
     * MyBatis mapper for error definitions table
     */
    private ErrorDefinitionsMapper errorDefinitionsMapper;

    /**
     * Instantiates a new Error definition dao.
     *
     * @param errorDefinitionsMapper the error definitions mapper
     */
    @Autowired
    public ErrorDefinitionDAOImpl(ErrorDefinitionsMapper errorDefinitionsMapper) {
        this.errorDefinitionsMapper = errorDefinitionsMapper;
    }

    @Override
    public Long getIdByData(ErrorDefinition errorDefinition) {
        return errorDefinitionsMapper.getErrorDefinitionIdByData(errorDefinition);
    }

    @Override
    public Long save(ErrorDefinition errorDefinition) {
        return errorDefinitionsMapper.insertErrorDefinition(errorDefinition);
    }

    @Override
    public ErrorDefinition find(Long errorDefinitionId) {
        return errorDefinitionsMapper.getById(errorDefinitionId);
    }

    @Override
    public List<Long> getDistinct() {
        return errorDefinitionsMapper.getDistinctErrorDefinitionIds();
    }

    @Override
    public List<Long> getAll() {
        return errorDefinitionsMapper.getAllErrorDefinitionIds();
    }

    @Override
    public Long delete(List<Long> errorDefinitionIds) {
        return errorDefinitionsMapper.deleteUnusedErrorDefinitions(errorDefinitionIds);
    }
}
