package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.ErrorMetadataMapper;

/**
 * Error Metadata DAO implementation
 * @author jkovac
 */
@Repository
public class ErrorMetadataDAOImpl implements ErrorMetadataDAO {

    /**
     * MyBatis mapper
     */
    private ErrorMetadataMapper errorMetadataMapper;

    /**
     * Instantiates a new Error metadata dao.
     *
     * @param errorMetadataMapper the error metadata mapper
     */
    @Autowired
    public ErrorMetadataDAOImpl(ErrorMetadataMapper errorMetadataMapper) {
        this.errorMetadataMapper = errorMetadataMapper;
    }

    @Override
    public ErrorMetadata find(Long id) {
        return errorMetadataMapper.getById(id);
    }

    @Override
    public Long getIdByData(ErrorMetadata errorMetadata) {
        return errorMetadataMapper.getErrorIdByData(errorMetadata);
    }

    @Override
    public Long save(ErrorMetadata errorMetadata) {
        return errorMetadataMapper.insertErrorMetadata(errorMetadata);
    }

    @Override
    public List<Long> getAllIds() {
        return errorMetadataMapper.getAllErrorMetadataIds();
    }

    @Override
    public List<Long> getDistinctIds() {
        return errorMetadataMapper.getDistinctErrorMetadataIds();
    }

    @Override
    public Long delete(List<Long> errorIds) {
        return errorMetadataMapper.deleteUnusedErrorMetadata(errorIds);
    }

}
