package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageUploadDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.PackageMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.PackageUploadMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.PruneMapper;

/**
 * Implementation of uploaded packages DAO
 * @author jkovac
 */
@Repository
public class PackageUploadDAOImpl implements PackageUploadDAO {


    /**
     * MyBatis package mapper
     */
    private PackageUploadMapper packageUploadMapper;

    /**
     * Ctr for uploaded packages DAO
     * @param packageUploadMapper MyBatis package mapper
     */
    @Autowired
    public PackageUploadDAOImpl(PackageUploadMapper packageUploadMapper) {
        this.packageUploadMapper = packageUploadMapper;
    }

    @Override
    public Long save(UploadedPackage uploadedPackage) {
        return packageUploadMapper.insertNewPackage(uploadedPackage);
    }

    @Override
    public List<ErrorDefinition> readErrorDefinitions(String absolutePath) {
        return packageUploadMapper.readErrorDefinitions(absolutePath);
    }

    @Override
    public List<UploadedLog> readLogs(String path) {
        return packageUploadMapper.readLogs(path);
    }

    @Override
    public Long upload(UploadedLog log, Long errorId, String packageName) {
        return packageUploadMapper.uploadError(log, errorId, packageName);
    }

    @Override
    public Long upload(UploadedLog log, String packageName) {
        return packageUploadMapper.uploadDebug(log, packageName);
    }
}
