package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl;

import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ScenarioExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.ScenarioExecutionsMapper;

/**
 * Scenario executions DAO implementation
 * @author jkovac
 */
@Repository
public class ScenarioExecutionDAOImpl implements ScenarioExecutionDAO {

    /**
     * MyBatis mapper
     */
    private ScenarioExecutionsMapper scenarioExecutionsMapper;

    /**
     * Instantiates a new Scenario execution dao.
     *
     * @param scenarioExecutionsMapper the scenario executions mapper
     */
    @Autowired
    public ScenarioExecutionDAOImpl(ScenarioExecutionsMapper scenarioExecutionsMapper) {
        this.scenarioExecutionsMapper = scenarioExecutionsMapper;
    }

    @Override
    public Optional<ScenarioExecution> find(String scenarioExecutionId) {
        return Optional.ofNullable(scenarioExecutionsMapper.getScenarioById(scenarioExecutionId));
    }

    @Override
    public Long save(ScenarioExecution scenarioExecution) {
        return scenarioExecutionsMapper.insertScenarioExecution(scenarioExecution);
    }

    @Override
    public List<ScenarioExecution> getAllFor(String workflowExecutionId) {
        return scenarioExecutionsMapper.getAllScenarioExecutions(workflowExecutionId);
    }

    @Override
    public Long delete(String scenarioExecutionId) {
        return scenarioExecutionsMapper.deleteScenarioExecution(scenarioExecutionId);
    }


}
