package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl;

import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.WorkflowExecutionsMapper;

/**
 * Workflow executions DAO implementation
 * @author jkovac
 */
@Repository
public class WorkflowExecutionDAOImpl implements WorkflowExecutionDAO {

    /**
     * MyBatis prune mapper
     */
    private WorkflowExecutionsMapper workflowExecutionsMapper;

    /**
     * Instantiates a new Workflow execution dao.
     *
     * @param workflowExecutionsMapper the workflow executions mapper
     */
    @Autowired
    public WorkflowExecutionDAOImpl(WorkflowExecutionsMapper workflowExecutionsMapper) {
        this.workflowExecutionsMapper = workflowExecutionsMapper;
    }

    @Override
    public Optional<WorkflowExecution> find(String workflowExecutionId) {
        return Optional.ofNullable(workflowExecutionsMapper.getWorkflowById(workflowExecutionId));
    }

    @Override
    public Long save(WorkflowExecution workflowExecution) {
        return workflowExecutionsMapper.insertWorkflowExecution(workflowExecution);
    }

    @Override
    public List<WorkflowExecution> getAll() {
        return workflowExecutionsMapper.getAllWorkflowExecutions();
    }

    @Override
    public Long delete(WorkflowExecution workflowExecution) {
        return workflowExecutionsMapper.deleteWorkflow(workflowExecution);
    }
}
