package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;

/**
 * MyBatis mapper for error definitions table queries
 * @author jkovac
 */
public interface ErrorDefinitionsMapper {

    /**
     * Get definition by ID
     * @param id DB ID
     * @return error definition stored under the given ID
     */
    ErrorDefinition getById(Long id);

    /**
     * Get DB id by data
     * @param errorDefinition wanted definition
     * @return ID
     */
    Long getErrorDefinitionIdByData(ErrorDefinition errorDefinition);

    /**
     * Persist a new error definition
     * @param errorDefinition definition to be inserted
     * @return amount of inserted rows
     */
    Long insertErrorDefinition(ErrorDefinition errorDefinition);

    /**
     * Get distinct ids that are currently used by the error metadata table
     * @return List of IDs of distinct definitions
     */
    List<Long> getDistinctErrorDefinitionIds();

    /**
     * Get all error definitions
     * @return List of ids of all stored definitions
     */
    List<Long> getAllErrorDefinitionIds();

    /**
     * Delete all definitions stored under given ids
     * @param errorDefinitionIds list of ids to delete
     * @return amount of deleted rows
     */
    Long deleteUnusedErrorDefinitions(@Param("errorDefinitionIds") List<Long> errorDefinitionIds);
}
