package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;

/**
 * MyBatis mapper for error metadata queries
 * @author jkovac
 */
public interface ErrorMetadataMapper {

    /**
     * Find metadata by id
     * @param id id of the wanted metadata
     * @return metadata stored under the given id
     */
    ErrorMetadata getById(Long id);

    /**
     * Get id of the given metadata
     * @param errorMetadata wanted metadata
     * @return id of the metadata
     */
    Long getErrorIdByData(ErrorMetadata errorMetadata);

    /**
     * Insert a new error metadata object to repository
     * @param error error metadata
     * @return amount of rows inserted
     */
    Long insertErrorMetadata(ErrorMetadata error);

    /**
     * @return List of all metadata IDs
     */
    List<Long> getAllErrorMetadataIds();

    /**
     * Get list of distinct error metadata currently used in the logs table, used for automated pruning
     * @return list of distinct error metadata ids in logs
     */
    List<Long> getDistinctErrorMetadataIds();

    /**
     * Delete given error metadata
     * @param errorIds ids to delete
     * @return amount of rows deleted
     */
    Long deleteUnusedErrorMetadata(@Param("errorIds") List<Long> errorIds);
}
