package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;

/**
 * MyBatis mapper for full text queries
 * @author jkovac
 */
public interface FullTextMapper {

    /**
     * Executes full text search over user message and technical message of error logs.
     * @param fullTextString String inputted by a user.
     * @param filter request filter
     * @param idx full text indices
     * @return Error logs that fit the search parameters
     */
    List<ErrorMetadata> performFullTextOverErrorLogs(@Param("fullTextString") String fullTextString, @Param("idx") List<Long> idx, @Param("filter") Filter filter);

    /**
     * Get Lucene indices fitting the query
     * @param query query passed by the used
     * @return log indices in '(id)' format
     */
    List<String> getIndicesFittingTheQuery(String query);
}
