package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.LogMetadata;

/**
 * MyBatis mapper for log context queries
 * @author jkovac
 */
public interface LogContextMapper {

    /**
     * Retrieve log metadata for the root log
     * @param logId id of the root log
     * @return root log metadata
     */
    LogMetadata getLogMetadata(@Param("logId") Long logId);

    /**
     * Get log with a given ID
     * @param logId id of the wanted log
     * @param logMetadata metadata of the log used to dynamically choose the correct table to join
     * @return log
     */
    Log getLog(@Param("logId") Long logId, @Param("metadata") LogMetadata logMetadata);

    /**
     * Load logs that occurred BEFORE the wanted log that have the same metaata and occurred in the same thread
     * @param log root log
     * @param logId id of the root log
     * @param offset amount of logs already loaded
     * @param amount amount to load
     * @param logMetadata root log metadata
     * @return list of logs that fit the criteria
     */
    List<Log> loadEarlier(@Param("log") Log log, @Param("logId") Long logId, @Param("offset") Long offset, @Param("amount") Long amount, @Param("metadata") LogMetadata logMetadata);

    /**
     * Load logs that occurred AFTER the wanted log
     * @param log root log
     * @param logId id of the root log
     * @param offset amount of logs already loaded
     * @param amount amount to load
     * @param logMetadata root log metadata
     * @return list of logs that fit the criteria
     */
    List<Log> loadLater(@Param("log") Log log, @Param("logId") Long logId, @Param("offset") Long offset, @Param("amount") Long amount, @Param("metadata") LogMetadata logMetadata);

}
