package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.LogMetadata;

/**
 * MyBatis mapper for basic log queries
 * @author jkovac
 */
public interface LogMapper {
    /**
     * Inserts new error log to logs table.
     * @param errorLog error log to be inserte
     * @param connDbId connection db id
     * @param errorId error metadata id
     * @return number of rows inserted
     */
    Long insertError(@Param("log") ErrorLog errorLog, @Param("errorId") Long errorId, @Param("connDbId") Long connDbId);

    /**
     * Inserts a new debug log to logs table.
     * @param debugLog debug log to be inserted
     * @param connDbId connection db id
     * @return number of rows inserted
     */
    Long insertDebug(@Param("log") DebugLog debugLog, @Param("connDbId") Long connDbId);

    /**
     * Prune logs that were emitted from a given application
     * @param application application from which should the logs be deleted
     * @return amount of logs pruned
     */
    Integer pruneLogsFromApplication(String application);

    /**
     * Deletes expired logs that do not belong to any execution.
     * @param pruneAfter time period, after which logs expire
     * @return number of logs deleted
     */
    Integer pruneExpiredNonExecutionLogs(double pruneAfter);

    /**
     * Retrieved all stored logs, used by tests
     * @return all stored logs
     */
    List<? extends Log> getAll();
}
