package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.PackageMetadata;

/**
 * MyBatis mapper for package create queries
 * @author jkovac
 */
public interface PackageCreateMapper {

    /**
     * Creates CSV dump of logs for a given application.
     * @param application application of which logs should be exported
     * @param path file path to the dump
     */
    void createDumpForApplication(@Param("application") String application, @Param("path")  String path);

    /**
     * Creates a CSV dump for a queried scenario execution.
     * @param connDbId connection db id
     * @param path path to the dump
     * @return number of rows in the file dump
     */
    Integer createDumpForConnection(@Param("connDbId") Long connDbId, @Param("path") String path);

    /**
     * Create dump of given error metadata
     * @param errorIds ids of the wanted metadata
     * @param path path to the dump
     */
    void createErrorDefinitionsDump(@Param("errorIds") List<Long> errorIds, @Param("path") String path);

    /**
     * Get error metadata ids for a given application
     * @param application wanted application
     * @return distinct error metadata ids emitted by the application
     */
    List<Long> getErrorIdsForApplication(@Param("application") String application);

    /**
     * Get error ids for the connection
     * @param connDbId db id of the connection
     * @return id list of the errors used by the connection
     */
    List<Long> getErrorIdsForConnection(@Param("connDbId") Long connDbId);

    /**
     * Build package metadata for a given connection (scenario, phase, ...)
     * @param connection packaged connection
     * @return new package metadata
     */
    PackageMetadata getPackageMetadata(@Param("connection") Connection connection);
}
