package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;

/**
 * MyBatis mapper for package upload queries
 * @author jkovac
 */
public interface PackageUploadMapper {

    /**
     * Insert metadata about the new uploaded package.
     * @param uploadedPackage metadata about the uploaded package
     * @return amount of inserted rows
     */
    Long insertNewPackage(UploadedPackage uploadedPackage);

    /**
     * Read CSV error definition dump
     * @param path path to the dump
     * @return read error definitions
     */
    List<ErrorDefinition> readErrorDefinitions(@Param("path") String path);

    /**
     * Read CSV log dump
     * @param path path to the dump
     * @return read logs
     */
    List<UploadedLog> readLogs(@Param("path") String path);

    /**
     * Insert a new error log belonging to a package
     * @param log log to be inserted
     * @param errorId error metadata id
     * @param packageName name of the uploaded package
     * @return amount of inserted rows
     */
    Long uploadError(@Param("log") UploadedLog log, @Param("errorId") Long errorId, @Param("packageName") String packageName);

    /**
     * Insert a new debug log belonging to a package
     * @param log log to be inserted
     * @param packageName package name
     * @return amount of inserted rows
     */
    Long uploadDebug(@Param("log") UploadedLog log, @Param("packageName") String packageName);
}
