package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;

/**
 * MyBatis mapper for operations regarding log deletion.
 * @author jkovac
 */
public interface PruneMapper {

    /**
     * Get amount of logs that would be deleted that don't belong to a workflow execution
     * @param pruneAfter new settings
     * @return amount of non-workflow logs deleted
     */
    Integer getAmountOfPotentiallyDeletedLogs(double pruneAfter);

    /**
     * Get workflow executions that would be deleted if settings were changed to a given value
     * @param pruneAfter new settings
     * @return workflow executions potentially deleted
     */
    List<WorkflowExecution> getAmountOfPotentiallyDeletedWorkflows(double pruneAfter);

    /**
     * Get amount of logs that would be deleted that belong to passed workflow executions
     * @param pruneAfter new settings
     * @param workflowExecutions potentially deleted workflow executions
     * @return amount of logs deleted
     */
    Integer getAmountOfPotentiallyDeletedWorkflowLogs(@Param("pruneAfter") double pruneAfter, @Param("workflowExecutions") List<WorkflowExecution> workflowExecutions);
}
