package eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;

/**
 * MyBatis mapper for scenario executions table queries
 * @author jkovac
 */
public interface ScenarioExecutionsMapper {

    /**
     * Lists all scenario executions for a given workflow execution ID.
     * @param workflowExecutionId ID of the workflow execution
     * @return list of the found scenario executions
     */
    List<ScenarioExecution> getAllScenarioExecutions(@Param("workflowExecutionId") String workflowExecutionId);

    /**
     * Delete a scenario execution
     * @param scenarioExecutionId id of the scenario execution
     * @return amount of deleted rows
     */
    Long deleteScenarioExecution(@Param("scenarioExecutionId") String scenarioExecutionId);

    /**
     * Persists a new scenario execution
     * @param scenarioExecution scenario execution that should be persisted
     * @return amount of inserted rows
     */
    Long insertScenarioExecution(ScenarioExecution scenarioExecution);

    /**
     * Queries scenario by its execution id
     * @param scenarioId id of the queried scenario
     * @return queried scenario execution
     */
    ScenarioExecution getScenarioById(String scenarioId);
}
