package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services;

import java.io.File;

/**
 * Service that handles log package creation.
 * @author jkovac
 */
public interface CreatePackageService {

    /**
     * Create package for a given connection that was used during a scenario execution
     *
     * The package has the following structure:
     *
     * {connection_name}_logs.csv
     * {connection_name}_errors.csv
     * {connection_name}_metadata.json
     *
     * @param workflowExecutionId workflow execution ID
     * @param scenarioExecutionId scenario executino ID
     * @param connectionId connection ID
     * @return package with dumps and metadata
     */
    File createLogsPackageForConnection(String workflowExecutionId, String scenarioExecutionId, String connectionId);


    /**
     * Creates a zip file containing csv dump from the database, that has all logs stored for a given application.
     *
     * The package has the following structure:
     *
     * application.json
     * {application_name}_errors.csv
     * {application_name}_logs.csv
     *
     * @param application Application for which the dump should be performed
     * @return file containing csv dump
     */
    File createLogsPackageForApplication(String application);
}
