package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services;

/**
 * Service providing force prune functionality for stored executions
 * @author jkovac
 */
public interface ForcePruneService {

    /**
     * Prune a workflow execution and with it all scenario executions and connections and all logs belonging to them
     * @param workflowExecutionId id of the deleted workflow execution
     */
    void forcePruneWorkflow(String workflowExecutionId);

    /**
     * Prune logs from an application
     * @param application name of the application
     */
    void forcePruneApplication(String application);

    /**
     * Prune uploaded package (only way to delete it)
     * @param packageName name of the deleted package
     */
    void pruneUploadedPackage(String packageName);

    /**
     * Prune a scenario execution and with it all used connections and logs
     * @param scenarioExecutionId scenario execution id
     */
    void forcePruneScenarioExecution(String scenarioExecutionId);

    /**
     * Prune a connection used in a scenario execution and with it all its logs
     * @param workflowExecutionId workflow execution id
     * @param scenarioExecutionId scenario execution id
     * @param connectionId connection ID
     */
    void forcePruneConnection(String workflowExecutionId, String scenarioExecutionId, String connectionId);

}
