package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;

/**
 * Service providing log context
 * @author jkovac
 */
public interface LogContextService {

    /**
     * Load logs that occurred AFTER the wanted log
     * @param logId id of the root log
     * @param offset amount of logs already loaded
     * @param amount amount to load
     * @return list of logs that fit the criteria
     */
    List<? extends Log> loadLater(Long logId, long offset, long amount);

    /**
     * Load logs that occurred BEFORE the wanted log that have the same metaata and occurred in the same thread
     * @param logId id of the root log
     * @param offset amount of logs already loaded
     * @param amount amount to load
     * @return list of logs that fit the criteria
     */
    List<? extends Log> loadEarlier(Long logId, long offset, long amount);

    /**
     * Get logs that happened before and after the given log
     * @param logId if of the root log
     * @return root log context
     */
    List<? extends Log> getContextForLog(Long logId);

}
