
package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.executions.WorkflowExecutionDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.DistinctErrorType;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;

/**
 * Service providing metadata regarding the repository
 * @author jkovac
 */
public interface MetadataProviderService {

    /**
     * List metadata about all executions stored in the repository.
     * @return list of workflow executions each containing information about its scenario executions
     * which contains names of its used connections
     */
    List<WorkflowExecutionDTO> listExecutions();

    /**
     * Distinct stored applications
     * @return distinct applications
     */
    List<String> getDistinctApplications();

    /**
     * Distinct categories fitting the filter settings
     * @param filter filter settings
     * @return distinct categories fitting the criteria
     */
    List<String> getDistinctCategories(Filter filter);

    /**
     * Distinct error types fitting the filter settings
     * @param filter filter settings
     * @return distinct error types
     */
    List<DistinctErrorType> getDistinctErrorTypes(Filter filter);

    /**
     * Get all uploaded packages
     * @return uploaded packages
     */
    List<UploadedPackage> getAllUploadedPackages();
}
