package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ScheduledFuture;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.PruneServiceImpl;

/**
 * Service handling scheduling of log pruning.
 * @author jkovac
 */
public interface PruneService {
    /**
     * Initial prune scheduling. Loops all stored workflow executions, calculates time,
     * when they should be pruned, and schedules tasks for pruning.
     */
    void schedulePruning();

    /**
     * Cancels all pruning tasks.
     */
    void dropPruning();

    /**
     * Spring scheduled method pruning expired logs that don't belong to a workflow execution. Cron task scheduled to run every hour.
     */
    void pruneNonExecutionLogs();

    /**
     * Drop pruning for a given workflow execution
     * @param workflowExecution workflow execution
     */
    void dropPruningFor(WorkflowExecution workflowExecution);

    /**
     * Schedules pruning for a single workflow execution. Used for newly created workflow executions.
     * @param workflowExecution workflow execution that should be pruned
     */
    void schedulePruning(WorkflowExecution workflowExecution);

    /**
     * Changes times of the scheduled prunings. Used for changed pruning settings.
     * @param updatedValue new pruning expiry value
     */
    void updateScheduledPruning(double updatedValue);

    /**
     * Get currently scheduled pruning tasks
     * @return pruning tasks
     */
    Map<PruneServiceImpl.PruneTask, ScheduledFuture<?>> getScheduledTasks();

    /**
     * Creates a new pruning task, which is then registered in the spring task scheduler.
     * @param workflowExecution workflow execution that should be pruned
     * @param date time of pruning
     * @return pruning task
     */
    PruneServiceImpl.PruneTask newPruningTask(WorkflowExecution workflowExecution, Date date);
}
