package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services;


import java.util.List;
import java.util.stream.Collectors;

import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;

/**
 * Common ancestor for all services providing pruning functionality.
 * @author jkovac
 */
public abstract class PruningCommon {

    /**
     * Error metadata DAO
     */
    private ErrorMetadataDAO errorMetadataDAO;

    /**
     * Error Definition DAO
     */
    private ErrorDefinitionDAO errorDefinitionDAO;

    /**
     * Instantiates a new Pruning common.
     *
     * @param errorMetadataDAO   the error metadata dao
     * @param errorDefinitionDAO the error definition dao
     */
    public PruningCommon(ErrorMetadataDAO errorMetadataDAO, ErrorDefinitionDAO errorDefinitionDAO) {
        this.errorMetadataDAO = errorMetadataDAO;
        this.errorDefinitionDAO = errorDefinitionDAO;
    }

    /**
     * Update error metadata and error definitions
     */
    protected void updateErrors(){
        updateErrorMetadata();
        updateErrorDefinitions();
    }

    /**
     * Find unused error metadata and prune it (i.e. scenario execution logs were pruned therefore this will delete its metadata)
     */
    private void updateErrorMetadata(){
        List<Long> allErrorMetadataIds = errorMetadataDAO.getAllIds();
        List<Long> allUsedErrorMetadata = errorMetadataDAO.getDistinctIds();

        List<Long> unusedErrorMetadata = allErrorMetadataIds
                .stream()
                .filter(e -> !allUsedErrorMetadata.contains(e))
                .collect(Collectors.toList())
                ;

        if (!unusedErrorMetadata.isEmpty()){
            errorMetadataDAO.delete(unusedErrorMetadata);
        }
    }

    /**
     * Find unused error definitions and prune them.
     */
    private void updateErrorDefinitions(){
        List<Long> allErrorDefinitionIds = errorDefinitionDAO.getAll();
        List<Long> allUsedErrorDefinitions = errorDefinitionDAO.getDistinct();

        List<Long> unusedErrorDefinitions = allErrorDefinitionIds
                .stream()
                .filter(e -> !allUsedErrorDefinitions.contains(e))
                .collect(Collectors.toList())
                ;

        if (!unusedErrorDefinitions.isEmpty()){
            errorDefinitionDAO.delete(unusedErrorDefinitions);
        }
    }
}
