package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ScenarioExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.ForcePruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruningCommon;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.WorkflowDoesNotExistException;

/**
 * Force prune service implementation
 * @author jkovac
 */
@Service
public class ForcePruneServiceImpl extends PruningCommon implements ForcePruneService {

    /**
     * Prune service used to drop pruning for the deleted workflow
     */
    private PruneService pruneService;

    /**
     * Scenario execution DAO
     */
    private ScenarioExecutionDAO scenarioExecutionDAO;

    /**
     * Connections DAO
     */
    private ConnectionsDAO connectionsDAO;

    /**
     * Package DAO
     */
    private PackageDAO packageDAO;

    /**
     * Workflow execution DAO
     */
    private WorkflowExecutionDAO workflowExecutionDAO;

    /**
     * Log DAO
     */
    private LogDAO logDAO;

    @Autowired
    public ForcePruneServiceImpl(WorkflowExecutionDAO workflowExecutionDAO,
                                 LogDAO logDAO,
                                 PruneService pruneService,
                                 ErrorDefinitionDAO errorDefinitionDAO,
                                 ErrorMetadataDAO errorMetadataDAO,
                                 ScenarioExecutionDAO scenarioExecutionDAO,
                                 ConnectionsDAO connectionsDAO,
                                 PackageDAO packageDAO) {
        super(errorMetadataDAO, errorDefinitionDAO);
        this.workflowExecutionDAO = workflowExecutionDAO;
        this.logDAO = logDAO;
        this.pruneService = pruneService;
        this.scenarioExecutionDAO = scenarioExecutionDAO;
        this.connectionsDAO = connectionsDAO;
        this.packageDAO = packageDAO;
    }

    @Override
    public void forcePruneWorkflow(String workflowExecutionId) {
        Optional<WorkflowExecution> workflowExecutionOpt = workflowExecutionDAO.find(workflowExecutionId);
        WorkflowExecution workflowExecution = workflowExecutionOpt.<WorkflowDoesNotExistException>orElseThrow(() -> {
            throw new WorkflowDoesNotExistException("Workflow with ID " + workflowExecutionId + " does not exist.");
        });

        pruneService.dropPruningFor(workflowExecution);
        workflowExecutionDAO.delete(workflowExecution);
        updateErrors();
    }

    @Override
    public void forcePruneApplication(String application) {
        logDAO.pruneLogsFromApplication(application);
        updateErrors();
    }

    @Override
    public void pruneUploadedPackage(String packageName) {
        packageDAO.delete(packageName);
        updateErrors();
    }

    @Override
    public void forcePruneScenarioExecution(String scenarioExecutionId) {
        scenarioExecutionDAO.delete(scenarioExecutionId);
        updateErrors();
    }

    @Override
    public void forcePruneConnection(String workflowExecutionId, String scenarioExecutionId, String connectionId) {
        Connection connection = new Connection(
                null,
                connectionId,
                scenarioExecutionId,
                null
        );

        connectionsDAO.delete(connection);
        updateErrors();
    }
}
