package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.FullTextSearchService;

/**
 * Service handling full text searching.
 * @author jkovac
 */
@Service
public class FullTextSearchServiceImpl implements FullTextSearchService {

    /**
     * DAO for log manipulation
     */
    private LogDAO logDAO;

    /**
     * Service facilitating full text searching
     * @param logDAO log DAO
     */
    @Autowired
    public FullTextSearchServiceImpl(LogDAO logDAO) {
        this.logDAO = logDAO;
    }

    @Override
    public List<ErrorDTO> fullText(String fullTextSearchString, Filter filter) {
        List<ErrorMetadata> errorMetadata = logDAO.fullText(fullTextSearchString, filter);
        List<ErrorDTO> errorDTOs = new ArrayList<>();

        errorMetadata.forEach(rawError -> {
            ErrorDefinition errorDef = rawError.getErrorDefinition();
            ErrorDTO errorDTO = new ErrorDTO(
                    rawError.getApplication(),
                    errorDef.getErrorCategory(),
                    errorDef.getErrorType(),
                    errorDef.getImpact(),
                    errorDef.getLogLevel(),
                    errorDef.getSolution(),
                    errorDef.getTechnicalMessage(),
                    errorDef.getUserMessage()
            );
            errorDTOs.add(errorDTO);
        });

        return errorDTOs;
    }

}
