package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogQueryService;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;

/**
 * Log Query service implementation
 * @author jkovac
 */
@Service
public class LogQueryServiceImpl implements LogQueryService {

    /**
     * Log DAO
     */
    private LogDAO logDAO;

    /**
     * Instantiates a new Log query service.
     *
     * @param logDAO the log dao
     */
    @Autowired
    public LogQueryServiceImpl(LogDAO logDAO) {
        this.logDAO = logDAO;
    }

    @Override
    @Transactional
    public List<ErrorDTO> queryLogs(Filter filter) {
        List<ErrorMetadata> errors = logDAO.queryLogs(filter);

        List<ErrorDTO> errorDTOs = new ArrayList<>();

        errors.forEach(rawError -> {
            ErrorDefinition errorDef = rawError.getErrorDefinition();
            ErrorDTO errorDTO = new ErrorDTO(
                    rawError.getApplication(),
                    errorDef.getErrorCategory(),
                    errorDef.getErrorType(),
                    errorDef.getImpact(),
                    errorDef.getLogLevel(),
                    errorDef.getSolution(),
                    errorDef.getTechnicalMessage(),
                    errorDef.getUserMessage()
            );
            errorDTOs.add(errorDTO);
        });

        return errorDTOs;
    }

}
