package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.executions.ScenarioExecutionDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.executions.WorkflowExecutionDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.DistinctErrorType;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.RepositoryMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ScenarioExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.MetadataProviderService;

/**
 * Metadata service implementation
 * @author jkovac
 */
@Service
public class MetadataProviderServiceImpl implements MetadataProviderService {

    /**
     * Package DAO
     */
    private PackageDAO packageDAO;

    /**
     * Scenario executino DAO
     */
    private ScenarioExecutionDAO scenarioExecutionDAO;

    /**
     * Workflow execution DAO
     */
    private WorkflowExecutionDAO workflowExecutionDAO;

    /**
     * Repository metadata DAO
     */
    private RepositoryMetadataDAO repositoryMetadataDAO;

    /**
     * Instantiates a new Metadata provider service.
     *
     * @param repositoryMetadataDAO the repository metadata dao
     * @param packageDAO            the package dao
     * @param scenarioExecutionDAO  the scenario execution dao
     * @param workflowExecutionDAO  the workflow execution dao
     */
    @Autowired
    public MetadataProviderServiceImpl(RepositoryMetadataDAO repositoryMetadataDAO,
                                       PackageDAO packageDAO,
                                       ScenarioExecutionDAO scenarioExecutionDAO,
                                       WorkflowExecutionDAO workflowExecutionDAO) {
        this.packageDAO = packageDAO;
        this.scenarioExecutionDAO = scenarioExecutionDAO;
        this.workflowExecutionDAO = workflowExecutionDAO;
        this.repositoryMetadataDAO = repositoryMetadataDAO;
    }

    @Override
    public List<WorkflowExecutionDTO> listExecutions() {
        List<WorkflowExecutionDTO> executions = new ArrayList<>();

        List<WorkflowExecution> workflowExecutions = workflowExecutionDAO.getAll();
        workflowExecutions.forEach(workflowExecution -> {

            List<ScenarioExecutionDTO> scenarioExecutionDTOs =
                    getScenarioExecutionMetadataForWorkflowExecution(workflowExecution.getWorkflowExecutionId());

            executions.add(
                    new WorkflowExecutionDTO(
                            workflowExecution.getWorkflowExecutionId(),
                            workflowExecution.getTimestamp(),
                            scenarioExecutionDTOs
                    )
            );
        });

        return executions;
    }

    private List<ScenarioExecutionDTO> getScenarioExecutionMetadataForWorkflowExecution(String workflowExecutionId){
        List<ScenarioExecutionDTO> scenarioExecutionDTOS = new ArrayList<>();

        List<ScenarioExecution> scenarioExecutions =
                scenarioExecutionDAO.getAllFor(workflowExecutionId);

        scenarioExecutions.forEach(scenarioExecution -> {
            List<String> connections =
                    repositoryMetadataDAO.getConnectionsForScenario(scenarioExecution.getScenarioExecutionId());
            scenarioExecutionDTOS.add(
                    new ScenarioExecutionDTO(
                            scenarioExecution.getScenarioExecutionId(),
                            scenarioExecution.getTechnology(),
                            scenarioExecution.getPhase(),
                            scenarioExecution.getScenarioName(),
                            connections
                    )
            );
        });

        return scenarioExecutionDTOS;
    }

    @Override
    public List<String> getDistinctApplications() {
        return repositoryMetadataDAO.getDistinctApplications();
    }

    @Override
    public List<String> getDistinctCategories(Filter filter) {
        return repositoryMetadataDAO.getDistinctCategories(filter);
    }

    @Override
    public List<DistinctErrorType> getDistinctErrorTypes(Filter filter) {
        return repositoryMetadataDAO.getDistinctErrorTypes(filter);
    }

    @Override
    public List<UploadedPackage> getAllUploadedPackages() {
        return packageDAO.getAll();
    }

}
