$(function() {
    
    // get packages
    $.ajax({
        type: 'get',
        dataType: 'json',
        url: '/manta-log-viewer/private/log-viewer/v1/packages',
        success: function(result){
            let packages = JSON.parse(JSON.stringify(result));
            packages.forEach(package => {
                $("#packagesList").append("<li type='package' selected='false' class='clickableListElement' id='" + package.packageName.split(' ').join('_') + "'>" + package.packageName + "</li>");
            });
        },
        fail: function(xhr, textStatus, errorThrown){
            console.log(textStatus + " " + errorThrown);
        }
    });

    $(document).on('click', '.clickableListElement', event => {
        $("#executionsDiv li").css("font-weight", "normal");
        $("#packagesDiv li").css("font-weight", "normal");

        let li = document.getElementById(event.target.id);

        if (li.getAttribute("selected") === "true"){
            li.setAttribute("selected", "false");
            printErrorMetadata("");
            return;
        }

        $("#userMessage").val("");
            $("#technicalMessage").val("");
            $("#solution").val("");
            $("#logMetadataContainer").css("visibility", "hidden");
            $("#exceptionRow").css("visibility", "hidden");
            $("#contextButtonContainer").css("visibility", "hidden");

        $("#executionsDiv li").each(function(index, element) {
            let el = document.getElementById(element.id);
            el.setAttribute("selected", "false");
        });

        $("#packagesDiv li").each(function(index, element) {
            let el = document.getElementById(element.id);
            el.setAttribute("selected", "false");
        });

        li.setAttribute("selected", "true");

        let type = event.target.type;

        $("#" + event.target.id).css("font-weight", "bold");

        let url = '';

        switch(type){
            case 'workflow':
                url += 'workflow-executions/';
                url += event.target.id;
                break;
            case 'scenario':
                url += 'workflow-executions/';
                let parentWorkflow = $('#' + event.target.id).attr('parent');
                url += parentWorkflow + '/';
                url += 'scenario-executions/';
                let id = event.target.id;
                url += id;
                break;
            case 'connection':
                let parentScenario = $(event.target).attr('parent');
                let parentWorkflowConn = $('#' + parentScenario).attr('parent');
                url += 'workflow-executions/';
                url += parentWorkflowConn + '/';
                url += 'scenario-executions/';
                url += parentScenario + '/';
                url += 'connections/';
                let connectionId = event.target.innerHTML;
                url += connectionId;
                break;
            case 'package':
                url += 'packages/';
                url += event.target.innerHTML;
                break;
        }    

        let data = getFilterParams();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url,
            traditional: true,
            data: {
                offset: 0,
                applications: data.applications,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorMetadata(result);
            }
        });
    });

    $.ajax({
        type: 'get',
        dataType: 'json',
        url: '/manta-log-viewer/private/log-viewer/v1/executions',
        success: function(result){
            printExecutions(result);
        }
    });

    $(document).on('click', '.collapseButton', event => {
        $(".errorMetadataTable tbody tr").css("background-color", "white");

        if (event.target.innerHTML === 'Collapse'){
            $(".logRow").each(function(index, element){
                let el = document.getElementById(element.id);
                let parent = el.getAttribute("parent");
                if (parent === event.target.parentElement.parentElement.id){
                    el.remove();
                }
            });

            $(".errorMetadataTable tbody tr").each(function(index, element) {
                let el = document.getElementById(element.id);
                el.setAttribute("selected", "false");
            });

            event.target.innerHTML = 'Expand';
            $("#" + event.target.parentElement.parentElement.id).css("background-color", "white");
            $("#logMetadataContainer").css("visibility", "hidden");
            $("#exceptionRow").css("visibility", "hidden");
            $("#contextButtonContainer").css("visibility", "hidden");
            $("#userMessage").val("");
            $("#technicalMessage").val("");
            $("#solution").val("");
            return;
        }

        event.target.innerHTML = 'Collapse';

        let errorContainer = $('.errorMetadataTable tbody');
        let errorId = event.target.parentElement.parentElement.id;
        let userMessage = errorContainer.data(errorId).userMessage;
        let technicalMessage = errorContainer.data(errorId).technicalMessage;
        let solution = errorContainer.data(errorId).solution;

        $("#userMessage").val(userMessage);
        $("#technicalMessage").val(technicalMessage);
        $("#solution").val(solution);

        let errorCategory = errorContainer.data(errorId).errorCategory;
        let collapsedElement = errorContainer.data(errorId).errorType;
        let application = errorContainer.data(errorId).application;

        let data = getFilterParams();
        let url = buildUrlFromSelectedExecutionOrPackage();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url + 'expand',
            traditional: true,
            data: {
                offset: 0,
                errorCategory: errorCategory,
                collapsedElement: collapsedElement,
                application: application,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorLogs(result, event);
            }
        });

    });


    $(document).on('click', '.errorMetadataItem td:not(.logCollapseButtonCell)', event => {
        let errorContainer = $('.errorMetadataTable tbody');

        $(".errorMetadataTable tbody tr").css("background-color", "white");

        let errorId = event.target.parentElement.id;

        let row = document.getElementById(errorId);

        if (row.getAttribute("selected") === "true"){
            row.setAttribute("selected", "false");
            $("#userMessage").val("");
            $("#technicalMessage").val("");
            $("#solution").val("");
            $("#logMetadataContainer").css("visibility", "hidden");
            $("#exceptionRow").css("visibility", "hidden");
            $("#contextButtonContainer").css("visibility", "hidden");
            return;
        }

        $("#logMetadataContainer").css("visibility", "hidden");
        $("#exceptionRow").css("visibility", "hidden");
        $("#contextButtonContainer").css("visibility", "hidden");

        $(".errorMetadataTable tbody tr").each(function(index, element) {
            let el = document.getElementById(element.id);
            el.setAttribute("selected", "false");
        });

        row.setAttribute("selected", "true");

        let userMessage = errorContainer.data(errorId).userMessage;
        let technicalMessage = errorContainer.data(errorId).technicalMessage;
        let solution = errorContainer.data(errorId).solution;

        $("#userMessage").val(userMessage);
        $("#technicalMessage").val(technicalMessage);
        $("#solution").val(solution);
        
        $("#" + errorId).css("background-color", "LightGrey");
    });

    $(document).on('click', '.logRow', event => {
        let logId = event.target.parentElement.id;
        
        let errorContainer = $('.errorMetadataTable tbody');

        $(".errorMetadataTable tbody tr").css("background-color", "white");

        let row = document.getElementById(logId);

        if (row.getAttribute("selected") === "true"){
            row.setAttribute("selected", "false");
            $("#userMessage").val("");
            $("#technicalMessage").val("");
            $("#solution").val("");
            $("#logMetadataContainer").css("visibility", "hidden");
            $("#exceptionRow").css("visibility", "hidden");
            $("#contextButtonContainer").css("visibility", "hidden");
            return;
        }

        $(".errorMetadataTable tbody tr").each(function(index, element) {
            let el = document.getElementById(element.id);
            el.setAttribute("selected", "false");
        });

        $("#userMessage").val(errorContainer.data(logId).userMessage);
        $("#technicalMessage").val(errorContainer.data(logId).technicalMessage);
        $("#solution").val(errorContainer.data(logId).solution);        
        $("#technologyText").html(errorContainer.data(logId).technology);        
        $("#loggerNameText").html(errorContainer.data(logId).loggerName);        
        $("#phaseText").html(errorContainer.data(logId).phase);        
        $("#timeText").html(errorContainer.data(logId).time);        
        $("#scenarioText").html(errorContainer.data(logId).scenario);        
        $("#threadText").html(errorContainer.data(logId).thread);        
        $("#connectionTest").html(errorContainer.data(logId).connectionId);        
        $("#contextText").html(errorContainer.data(logId).context);        
        $("#exceptionNameText").html(errorContainer.data(logId).exceptionName);        
        $("#stacktraceText").val(errorContainer.data(logId).stacktrace);
        
        row.setAttribute("selected", "true");

        $("#logMetadataContainer").css("visibility", "visible");
        $("#contextButtonContainer").css("visibility", "visible");

        if (errorContainer.data(logId).exceptionName){
            $("#exceptionRow").css("visibility", "visible");
        }

        $("#" + logId).css("background-color", "LightGrey");
    });

    $(document).on('click', '#createPackageBtn', event => {
        
        let selectedItem;
        
        $("#executionsDiv li").each(function(index, element) {
            let el = document.getElementById(element.id);
            let selected = el.getAttribute("selected");
            if (selected === "true"){
                selectedItem = el;
            }
        });

        $("#packagesDiv li").each(function(index, element) {
            let el = document.getElementById(element.id);
            let selected = el.getAttribute("selected");
            if (selected === "true"){
                selectedItem = el;
            }
        });

        if (selectedItem.type != 'connection'){
            alert('Please select a connection to package!');
            return;
        }

        let url = '';
        let parentScenario = $(selectedItem).attr('parent');
        let parentWorkflowConn = $('#' + parentScenario).attr('parent');
        url += 'workflow-executions/';
        url += parentWorkflowConn + '/';
        url += 'scenario-executions/';
        url += parentScenario + '/';
        url += 'connection/';
        let connectionId = selectedItem.innerHTML;
        url += connectionId  + '/';

        $.ajax({
            type: 'get',
            xhrFields: {
                responseType: 'blob'
            },
            url: '/manta-log-viewer/private/log-viewer/v1/' + url + 'package',
            success: function(result){
                var link=document.createElement('a');
                link.href=window.URL.createObjectURL(result);
                link.download=connectionId + ".zip";
                document.body.appendChild(link);
                link.click();
            }
        });
    });

    $(document).on('click', '#deleteSelectedBtn', event => {
        let url = buildUrlFromSelectedExecutionOrPackage();

        $.ajax({
            type: 'delete',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url,
            success: function(result){
                alert('Successfully deleted!');
                location.reload();
            }
        });
    });

    $("#uploadPackageForm").submit(function(event){
        event.preventDefault();
        
        let packageName = $('#uploadPackageName').val();

        let package = document.getElementById("packageFile").files[0];
        let formData = new FormData();

        formData.append("uploadedPackage", package);

        fetch('/manta-log-viewer/private/log-viewer/v1/upload?packageName=' + packageName, {method: "POST", body: formData})
        .then(data => {
            if (data.status === 200){
                alert(packageName + " was successfully uploaded!");
                location.reload();
            } else {
                alert("Unable to upload the package due to bad response!");
            }
        });
    });


    $("#ftForm").submit(function(event){
        event.preventDefault();
        let inputString = $('#ftSearchField').val();
        
        let data = getFilterParams();
        let url = buildUrlFromSelectedExecutionOrPackage();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url + '/search',
            traditional: true,
            data: {
                offset: 0,
                query: inputString,
                applications: data.applications,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorMetadata(result);
            }
        });
    });

    $(document).on('click', '#getContextBtn', event => {

        let logId;
        $('.logRow').each(function(index, element){
            let el = document.getElementById(element.id);
            let selected = el.getAttribute("selected");
            if (selected === "true"){
                logId = el.id.substring(3);
            }
        });

        if (!logId){
            alert('Unable to show context!');
            return;
        }

        $.ajax({
            type: 'get',
            url: '/manta-log-viewer/private/log-viewer/v1/logs/' + logId,
            data: {
                offset: 0
            },
            success: function(result){
                let dialog = "<div id='dialog' class='dialogWin' title='Context'>";
                dialog += "<button id='loadEarlierBtn'>Load earlier</button><br>";
                dialog += "<div id='contextContainer'>";
                dialog += buildContext(result);
                dialog += '</div>';
                dialog += "<button id='loadLaterBtn'>Load later</button>";
                dialog += '</div>';

                $('body').data("offset", {
                    earlierOffset: 5,
                    laterOffset: 5
                });
                $(dialog).dialog({
                    height: 800,
                    width: 1000,
                    closeText: '',
                    modal: true,
                    draggable: false,
                    closeOnEscape: false,
                    close: function () {
                        $(this).remove();
                    }
                });
            }
        });
    });

    $(document).on('click', '#loadEarlierBtn', event => {
        let logId;
        $('.logRow').each(function(index, element){
            let el = document.getElementById(element.id);
            let selected = el.getAttribute("selected");
            if (selected === "true"){
                logId = el.id.substring(3);
            }
        });

        if (!logId){
            alert('Unable to show context!');
            return;
        }

        let currOffset = $('body').data("offset").earlierOffset;
        let laterOffset = $('body').data("offset").laterOffset;

        $.ajax({
            type: 'get',
            url: '/manta-log-viewer/private/log-viewer/v1/logs/' + logId + '/before',
            data: {
                offset: currOffset
            },
            success: function(result){
                let contextContainer = $('#contextContainer');
                let context = contextContainer.html();
                context = buildContext(result) + context;
                contextContainer.html(context);
                $('body').data("offset", {
                    earlierOffset: currOffset + 5,
                    laterOffset: laterOffset
                });
            }
        });
    });

    $(document).on('click', '#loadLaterBtn', event => {
        let logId;
        $('.logRow').each(function(index, element){
            let el = document.getElementById(element.id);
            let selected = el.getAttribute("selected");
            if (selected === "true"){
                logId = el.id.substring(3);
            }
        });

        if (!logId){
            alert('Unable to show context!');
            return;
        }

        let currOffset = $('body').data("offset").laterOffset;
        let earlierOffset = $('body').data("offset").earlierOffset;

        $.ajax({
            type: 'get',
            url: '/manta-log-viewer/private/log-viewer/v1/logs/' + logId + '/after',
            data: {
                offset: currOffset
            },
            success: function(result){
                let contextContainer = $('#contextContainer');
                let context = contextContainer.html();
                context = context + buildContext(result);
                contextContainer.html(context);
                $('body').data("offset", {
                    earlierOffset: earlierOffset,
                    laterOffset: currOffset + 5
                });
            }
        });
    });

    $(document).on('click', "#applicationSetBtn", function(event){
        event.preventDefault();

        let url = buildUrlFromSelectedExecutionOrPackage();
        let data = getFilterParams();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url,
            traditional: true,
            data: {
                offset: 0,
                applications: data.applications,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorMetadata(result);
            }
        });
    });

    $(document).on('click', "#applicationUnsetBtn", function(event){
        event.preventDefault();

        $("#applicationsForm input:checked").each(function(index, element){
            element.checked = false;
        });

        let url = buildUrlFromSelectedExecutionOrPackage();
        let data = getFilterParams();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url,
            traditional: true,
            data: {
                offset: 0,
                applications: data.applications,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorMetadata(result);
            }
        });
    });

    $(document).on('click', "#categoriesSetBtn", function(event){
        event.preventDefault();
        
        let data = getFilterParams();
        let url = buildUrlFromSelectedExecutionOrPackage();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url,
            traditional: true,
            data: {
                offset: 0,
                applications: data.applications,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorMetadata(result);
            }
        });
    });

    $(document).on('click', "#categoriesUnsetBtn", function(event){
        event.preventDefault();

        $("#categoriesForm option:selected").each(function(index, element){
            element.selected = false;
        });
        
        let data = getFilterParams();
        let url = buildUrlFromSelectedExecutionOrPackage();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url,
            traditional: true,
            data: {
                offset: 0,
                applications: data.applications,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorMetadata(result);
            }
        });
    });

    $(document).on('click', "#errorTypesSetBtn", function(event){
        event.preventDefault();
        
        let data = getFilterParams();
        let url = buildUrlFromSelectedExecutionOrPackage();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url,
            traditional: true,
            data: {
                offset: 0,
                applications: data.applications,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorMetadata(result);
            }
        });
    });

    $(document).on('click', "#errorTypesUnsetBtn", function(event){
        event.preventDefault();
        
        $("#errorTypesForm option:selected").each(function(index, element){
            element.selected = false;
        });

        let data = getFilterParams();
        let url = buildUrlFromSelectedExecutionOrPackage();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/' + url,
            traditional: true,
            data: {
                offset: 0,
                applications: data.applications,
                categories: data.categories,
                errorTypes: data.errorTypes
            },
            success: function(result){
                printErrorMetadata(result);
            }
        });
    });

    $("#pruningForm").submit(function(event){
        event.preventDefault();

        let newExpiry = $('#changeSettingsInput').val();

        $.ajax({
            type: 'get',
            dataType: 'json',
            url: '/manta-log-viewer/private/log-viewer/v1/settings/set-log-expiry-check',
            data: {
                daysUntilExpiry: newExpiry
            },
            success: function(result){
                let response = JSON.parse(JSON.stringify(result));
                let workflowExecutionIds = [];
                response.workflowsPotentiallyPruned.forEach(workflowExecution => {
                    workflowExecutionIds.push(workflowExecution.workflowExecutionId);
                })
                let message = 'Are you sure you want to change the log expiration value to ' + newExpiry + '? This change would automatically delete ' + response.workflowLogsPotentiallyPruned + ' logs from following workflow executions:' + workflowExecutionIds + ' as well as ' + response.nonWorkflowLogsPotentiallyPruned +' logs that are not part of any execution.'; 
                
                if (confirm(message)){
                    let newExpiry2 = $('#changeSettingsInput').val();

                    $.ajax({
                        type: 'post',
                        dataType: 'json',
                        url: '/manta-log-viewer/private/log-viewer/v1/settings/set-log-expiry',
                        data: "daysUntilExpiry=" + newExpiry,
                        success: function(result){
                            alert("Log expiration time successfully changed to " + newExpiry2 + " days.");
                            location.reload();
                        }
                    });

                }
            },
            fail: function(result){
                alert(result);
            }
        });
    });
    
    getCategories();
    getErrorTypes();
    getApplications();
});

function getFilterParams(){
    let applications = [];
    $("#applicationsForm input:checked").each(function(){
        applications.push($(this).attr('value'));
    });

    let categories = [];
    $("#categoriesForm option:selected").each(function(){
        categories.push($(this).attr('value'));
    });

    let errorTypes = [];
    $("#errorTypesForm option:selected").each(function(){
        errorTypes.push($(this).attr('value'));
    });

    let data = {
        applications: applications,
        categories: categories,
        errorTypes: errorTypes
    }

    return data;
}

function buildContext(response){
    let logs = JSON.parse(JSON.stringify(response));
    
    let context = '';
    let breakTag = '<br>';

    logs.forEach(log => {
        if (log.type === 'DEBUG'){
            context += buildDebugLog(log);
        } else {
            context += buildErrorLog(log);
        }
        context += breakTag;
    });

    console.log(context);

    return context;
}

function buildDebugLog(log){
    const date = new Date(log.epochSecond * 1000);
    const year = date.getFullYear();
    const month = date.getMonth() + 1;
    const day = date.getDate();
    const hour = date.getHours();
    const minute = date.getMinutes();
    const sec = date.getSeconds();
    const nano = log.nanoSecond / 100000;

    let dateString = year + "/" + month + "/" + day + " " + hour + ":" + minute + ":" + sec + "." + nano;

    let thread = log.thread;
    let level = log.level;
    let logger = log.loggerName;
    let message = log.message
    let context = log.context;

    let completeLog = dateString + ' [' + thread + '] ' + level + ' ' + logger + ' ';
    if (context){
        completeLog += '[Context: ' + context + '] ';
    }
    completeLog += message;

    return completeLog;
}

function buildErrorLog(log){
    const date = new Date(log.epochSecond * 1000);
    const year = date.getFullYear();
    const month = date.getMonth() + 1;
    const day = date.getDate();
    const hour = date.getHours();
    const minute = date.getMinutes();
    const sec = date.getSeconds();
    const nano = log.nanoSecond / 100000;

    let dateString = year + "/" + month + "/" + day + " " + hour + ":" + minute + ":" + sec + "." + nano;

    let thread = log.thread;
    let level = log.level;
    let logger = log.loggerName;
    let message = log.message
    let context = log.context;

    let completeLog = '<br>' + dateString + ' [' + thread + '] ' + level + ' ' + logger + ' ';
    if (context){
        completeLog += '[Context: ' + context + '] ';
    }
    
    completeLog += '<br>';
    completeLog += log.category + ' ' + log.errorType;
    completeLog += '<br>';
    completeLog += 'User message: ' + log.userMessage + '<br>';
    completeLog += 'Technical message: ' + log.technicalMessage + '<br>';
    completeLog += 'Solution: ' + log.solution + '<br>';
    completeLog += 'Lineage impact: ' + log.lineageImpact + '<br>';

    return completeLog;
}

function printErrorLogs(response, event){
    let logs = JSON.parse(JSON.stringify(response));
    let errorId = event.target.parentElement.parentElement.id;
    $("#" + errorId).css("background-color", "LightGrey");

    let logRows = "";
    

    logs.forEach(log => {
        let logRow = "<tr parent='" + errorId +"' selected='false' class='logRow' id='" + "log" + log.logId + "'>";

        logRow += "<td>" + log.category +"</td>";
        logRow += "<td>" + log.errorType +"</td>";
        logRow += "<td>" + log.lineageImpact +"</td>";
        logRow += "<td>" + log.level +"</td>";
        logRow += "<td>" + log.application +"</td>";
        logRow += "<td></td>"
        logRow += '</tr>';

        let errorContainer = $('.errorMetadataTable tbody');

        const date = new Date(log.epochSecond * 1000);
        const year = date.getFullYear();
        const month = date.getMonth() + 1;
        const day = date.getDate();
        const hour = date.getHours();
        const minute = date.getMinutes();
        const sec = date.getSeconds();
        const nano = log.nanoSecond / 100000;

        let dateString = year + "/" + month + "/" + day + " " + hour + ":" + minute + ":" + sec + "." + nano;

        errorContainer.data("log" + log.logId, {
            userMessage: log.userMessage,
            technicalMessage: log.technicalMessage,
            solution: log.solution,
            technology: log.technology,
            phase: log.phase,
            loggerName: log.loggerName,
            scenarioName: log.scenario,
            thread: log.thread,
            connectionId: log.connectionId,
            context: log.context,
            exceptionName: log.exceptionName,
            stacktrace: log.extendedStacktrace,
            time: dateString
        });
        
        logRows += logRow;
    });

    $(logRows).insertAfter("#" + errorId);
}

function printErrorMetadata(errorMetadataResponse){
    let errorMetadata = JSON.parse(JSON.stringify(errorMetadataResponse));

    let errorContainer = $('.errorMetadataTable tbody');
    errorContainer.empty();
    errorContainer.removeData();

    let counter = 0;

    errorMetadata.forEach(error => {
        let errorId = 'error' + counter;

        let errorRow = "<tr selected='false' class='errorMetadataItem' id='" + errorId + "'>"

        errorRow += "<td>" + error.errorCategory +"</td>";
        errorRow += "<td>" + error.errorType +"</td>";
        errorRow += "<td>" + error.impact +"</td>";
        errorRow += "<td>" + error.logLevel +"</td>";
        errorRow += "<td>" + error.application +"</td>";
        errorRow += "<td class='logCollapseButtonCell'><button class='collapseButton'>Expand</button></td>";
    
        errorRow += '</tr>'

        errorContainer.append(errorRow);

        errorContainer.data(errorId, {
            errorCategory: error.errorCategory,
            errorType: error.errorType,
            application: error.application,
            userMessage: error.userMessage,
            technicalMessage: error.technicalMessage,
            solution: error.solution
        });

        counter += 1;
    });
}



function printExecutions(executions){
    let workflowExecutions = JSON.parse(JSON.stringify(executions));
    workflowExecutions.forEach(workflowExecution => {
        const date = new Date(workflowExecution.timestamp);
        const year = date.getFullYear();
        const month = date.getMonth() + 1;
        const day = date.getDate();
        const hour = date.getHours();
        const minute = date.getMinutes();
        const sec = date.getSeconds();

        let dateString = year + "/" + month + "/" + day + " " + hour + ":" + minute + ":" + sec;

        $("#workflowExecutionsList").append("<li selected='false' type='workflow' class='clickableListElement' id='" + workflowExecution.workflowExecutionId + "'>" + workflowExecution.workflowExecutionId + " " + dateString + "</li>");
        
        workflowExecution.scenarioExecutions.forEach(scenarioExecution => {
            let scenariosList = workflowExecution.workflowExecutionId + "List";

            // create scenartio executions sublist
            $('#workflowExecutionsList').append("<ul id='" + scenariosList + "'></ul>");

            // append the scenario itself
            $('#' + scenariosList).append("<li selected='false' type='scenario' parent='" + workflowExecution.workflowExecutionId + "' id='" + scenarioExecution.scenarioExecutionId + "' class='clickableListElement'>" + scenarioExecution.scenarioName + "</li>");

            if (scenarioExecution.connections.length > 0){
                let connectionsListName = scenarioExecution.scenarioExecutionId + "List";

                // created sublist of connections
                $('#' + scenariosList).append("<ul id='"  + connectionsListName + "'></ul>");

                // add connections to that list
                scenarioExecution.connections.forEach(connection => {
                    let connElementId = workflowExecution.workflowExecutionId + scenarioExecution.scenarioExecutionId + connection;
                    $('#' + connectionsListName).append("<li selected='false' type='connection' id='"+ connElementId +"' parent='" + scenarioExecution.scenarioExecutionId +"' class='clickableListElement'>" + connection + "</li>");
                });
            }
        })


    });
}

function buildUrlFromSelectedExecutionOrPackage(){
    let selectedItem;
        
    $("#executionsDiv li").each(function(index, element) {
        let el = document.getElementById(element.id);
        let selected = el.getAttribute("selected");
        if (selected === "true"){
            selectedItem = el;
        }
    });

    $("#packagesDiv li").each(function(index, element) {
        let el = document.getElementById(element.id);
        let selected = el.getAttribute("selected");
        if (selected === "true"){
            selectedItem = el;
        }
    });

    let url = '';

    if (!selectedItem){
        return url;
    }

    switch(selectedItem.type){
        case 'workflow':
            url += 'workflow-executions/';
            url += selectedItem.id  + '/';
            break;
        case 'scenario':
            url += 'workflow-executions/';
            let parentWorkflow = $('#' + selectedItem.id).attr('parent');
            url += parentWorkflow + '/';
            url += 'scenario-executions/';
            let id = selectedItem.id;
            url += id  + '/';
            break;
        case 'connection':
            let parentScenario = $(selectedItem).attr('parent');
            let parentWorkflowConn = $('#' + parentScenario).attr('parent');
            url += 'workflow-executions/';
            url += parentWorkflowConn + '/';
            url += 'scenario-executions/';
            url += parentScenario + '/';
            url += 'connections/';
            let connectionId = selectedItem.innerHTML;
            url += connectionId  + '/';
            break;
        case 'package':
            url += 'packages/';
            url += selectedItem.innerHTML + '/';
            break;
    }

    return url;
}

function getCategories(){
    $("#categoriesMultiselct").empty();
    $.ajax({
        type: 'get',
        dataType: 'json',
        url: '/manta-log-viewer/private/log-viewer/v1/categories',
        success: function(result){
            let categories = JSON.parse(JSON.stringify(result));
            categories.forEach(category => {
                $("#categoriesMultiselct").append("<option value='"+ category +"'>" + category + "</option>");
            });
        },
        fail: function(xhr, textStatus, errorThrown){
            console.log(textStatus + " " + errorThrown);
        }
    });
}

function getErrorTypes(){
    $("#errorTypesMultiselect").empty();

    $.ajax({
        type: 'get',
        dataType: 'json',
        url: '/manta-log-viewer/private/log-viewer/v1/errortypes',
        success: function(result){
            let errortypes = JSON.parse(JSON.stringify(result));
            errortypes.forEach(errortype => {
                $("#errorTypesMultiselect").append("<option value='" + errortype.errorType +"'>" + errortype.errorCategory + ": " + errortype.errorType + "</option>");
            });
        },
        fail: function(xhr, textStatus, errorThrown){
            console.log(textStatus + " " + errorThrown);
        }
    });
}

function getApplications(){
    $("#applicationsForm").empty();

    $.ajax({
        type: 'get',
        dataType: 'json',
        url: '/manta-log-viewer/private/log-viewer/v1/applications',
        success: function(result){
            let applications = JSON.parse(JSON.stringify(result));
            applications.forEach(application => {
                $("#applicationsForm").append("<input class='applicationCheckbox' type='checkbox' name='" + application +"' value='" + application + "'/>");
                $("#applicationsForm").append("<label for='"+ application + "'>" + application + "</label><br>");
            });
        },
        fail: function(xhr, textStatus, errorThrown){
            console.log(textStatus + " " + errorThrown);
        }
    });
}

