package eu.profinit.manta.bachelor.thesis.logging.api.logging;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.MarkerManager;

/**
 * Base logging class serving as a wrapper to log4j logger that extends its functionality with
 * logging of categorized errors.
 *
 * @author jkovac
 */
public final class Logger {
    /**
     * Log4j logger belonging to a given class.
     */
    private final org.apache.logging.log4j.Logger logger;

    /**
     * Marker for TRACE, DEBUG and INFO logs.
     */
    private final Marker DEBUG_MARKER = MarkerManager.getMarker("DEBUG");

    /**
     * Creates a new Manta logger for a given class.
     * @param className Name of the given class.
     */
    public Logger(String className) {
        logger = LogManager.getLogger(className);
    }

    /**
     * Creates a new Manta logger for a given class.
     * @param clazz The given class.
     */
    public Logger(Class clazz) {
        logger = LogManager.getLogger(clazz);
    }

    /**
     * ------------------------------------- TRACE LEVEL -------------------------------------
     */

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the object to log
     */
    public void trace(final Object message) {
        logger.trace(DEBUG_MARKER, message);
    }

    /**
     * Logs a message with an exception at trace level.
     *
     * @param message the object to log
     * @param t the exception to log
     */
    public void trace(final Object message, final Throwable t) {
        logger.trace(DEBUG_MARKER, message, t);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     */
    public void trace(final String message) {
        logger.trace(DEBUG_MARKER, message);
    }

    /**
     * Logs a message with an exception at trace level.
     *
     * @param message the message to log
     * @param t the exception to log
     */
    public void trace(final String message, final Throwable t) {
        logger.trace(DEBUG_MARKER, message, t);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     */
    public void trace(final String message, final Object arg1) {
        logger.trace(DEBUG_MARKER, message, arg1);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2, final Object arg3) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2, arg3);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2, arg3, arg4);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6,
                                            final Object arg7) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6,
                                            final Object arg7, final Object arg8) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
    }


    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     * @param arg9 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6,
                                            final Object arg7, final Object arg8, final Object arg9) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
    }

    /**
     * Logs a message with parameters at trace level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     * @param arg9 parameter to the message.
     * @param arg10 parameter to the message.
     */
    public void trace(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6,
                                            final Object arg7, final Object arg8, final Object arg9,
                                            final Object arg10) {
        logger.trace(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param params parameters to the message.
     */
    public void trace(final String message, final Object... params) {
        logger.trace(DEBUG_MARKER, message, params);
    }

    /**
     * ------------------------------------- DEBUG LEVEL -------------------------------------
     */

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the object to log
     */
    public void debug(final Object message) {
        logger.debug(DEBUG_MARKER, message);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the object to log
     * @param t throwable to log.
     */
    public void debug(final Object message, final Throwable t) {
        logger.debug(DEBUG_MARKER, message, t);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     */
    public void debug(final String message) {
        logger.debug(DEBUG_MARKER, message);
    }

    /**
     * Logs a message with an exception at debug level.
     *
     * @param message the message to log
     * @param t throwable to log.
     */
    public void debug(final String message, final Throwable t) {
        logger.debug(DEBUG_MARKER, message, t);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     */
    public void debug(final String message, final Object arg1) {
        logger.debug(DEBUG_MARKER, message, arg1);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2, final Object arg3) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2, arg3);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2, arg3, arg4);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6,
                                            final Object arg7) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6,
                                            final Object arg7, final Object arg8) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     * @param arg9 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6,
                                            final Object arg7, final Object arg8, final Object arg9) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     * @param arg9 parameter to the message.
     * @param arg10 parameter to the message.
     */
    public void debug(final String message, final Object arg1, final Object arg2, final Object arg3,
                                            final Object arg4, final Object arg5, final Object arg6,
                                            final Object arg7, final Object arg8, final Object arg9,
                                            final Object arg10) {
        logger.debug(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
    }

    /**
     * Logs a message with parameters at debug level.
     *
     * @param message the message to log
     * @param params parameters to the message.
     */
    public void debug(final String message, final Object... params) {
        logger.debug(DEBUG_MARKER, message, params);
    }

    /**
     * ------------------------------------- INFO LEVEL -------------------------------------
     */

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the object to log
     */
    public void info(final Object message) {
        logger.info(DEBUG_MARKER, message);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the object to log
     * @param t exception to log.
     */
    public void info(final Object message, final Throwable t) {
        logger.info(DEBUG_MARKER, message, t);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     */
    public void info(final String message) {
        logger.info(DEBUG_MARKER, message);
    }

    /**
     * Logs a message with an exception at info level.
     *
     * @param message the message to log
     * @param t exception to log.
     */
    public void info(final String message, final Throwable t) {
        logger.info(DEBUG_MARKER, message, t);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     */
    public void info(final String message, final Object arg1) {
        logger.info(DEBUG_MARKER, message, arg1);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2) {
        logger.info(DEBUG_MARKER, message, arg1, arg2);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2, final Object arg3) {
        logger.info(DEBUG_MARKER, message, arg1, arg2, arg3);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2, final Object arg3,
                                           final Object arg4) {
        logger.info(DEBUG_MARKER, message, arg1, arg2, arg3, arg4);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2, final Object arg3,
                                           final Object arg4, final Object arg5) {
        logger.info(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2, final Object arg3,
                                           final Object arg4, final Object arg5, final Object arg6) {
        logger.info(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2, final Object arg3,
                                           final Object arg4, final Object arg5, final Object arg6,
                                           final Object arg7) {
        logger.info(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2, final Object arg3,
                                           final Object arg4, final Object arg5, final Object arg6,
                                           final Object arg7, final Object arg8) {
        logger.info(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     * @param arg9 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2, final Object arg3,
                                           final Object arg4, final Object arg5, final Object arg6,
                                           final Object arg7, final Object arg8, final Object arg9) {
        logger.info(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log
     * @param arg1 parameter to the message.
     * @param arg2 parameter to the message.
     * @param arg3 parameter to the message.
     * @param arg4 parameter to the message.
     * @param arg5 parameter to the message.
     * @param arg6 parameter to the message.
     * @param arg7 parameter to the message.
     * @param arg8 parameter to the message.
     * @param arg9 parameter to the message.
     * @param arg10 parameter to the message.
     */
    public void info(final String message, final Object arg1, final Object arg2, final Object arg3,
                                           final Object arg4, final Object arg5, final Object arg6,
                                           final Object arg7, final Object arg8, final Object arg9,
                                           final Object arg10) {
        logger.info(DEBUG_MARKER, message, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
    }

    /**
     * Logs a message with parameters at info level.
     *
     * @param message the message to log.
     * @param params parameters to the message.
     */
    public void info(final String message, final Object... params) {
        logger.info(DEBUG_MARKER, message, params);
    }

    /**
     * ----------------------------------- WARN & ERROR & FATAL -----------------------------------
     */

    /**
     * Logs a message with parameters at warn, error and fatal level.
     *
     * @param builder builder with error messages, its placeholders and other params.
     */
    public void log(GenericBuilder builder) {
        log(builder, false);
    }

    /**
     * Logs a message with parameters at warn, error and fatal level. If there is a set context, it is ignored.
     *
     * @param builder builder with error messages, its placeholders and other params.
     * @param contextFlag true if context should be ignored.
     */
    public void log(GenericBuilder builder, boolean contextFlag){
        builder.log(logger, contextFlag);
    }
}
