package eu.profinit.manta.bachelor.thesis.logging.api.logging;

import java.util.HashMap;
import java.util.Map;

import org.apache.logging.log4j.ThreadContext;

/**
 * Class injecting additional data to logs.
 * @author jkovac
 */
public final class LoggingContext {

    /**
     * Returns map of context values.
     * @return map of context values.
     */
    public static Map<ContextLevel, String> getContextValues(){
        Map<ContextLevel, String> values = new HashMap<>();

        values.put(ContextLevel.WORKFLOW_ID, ThreadContext.get(ContextLevel.WORKFLOW_ID.name()));
        values.put(ContextLevel.EXECUTION_ID, ThreadContext.get(ContextLevel.EXECUTION_ID.name()));
        values.put(ContextLevel.TECHNOLOGY, ThreadContext.get(ContextLevel.TECHNOLOGY.name()));
        values.put(ContextLevel.CONNECTION_ID, ThreadContext.get(ContextLevel.CONNECTION_ID.name()));
        values.put(ContextLevel.PHASE, ThreadContext.get(ContextLevel.PHASE.name()));
        values.put(ContextLevel.SCENARIO, ThreadContext.get(ContextLevel.SCENARIO.name()));
        values.put(ContextLevel.CONTEXT, ThreadContext.get(ContextLevel.CONTEXT.name()));

        return values;
    }

    /**
     * Put technology name to the context.
     * @param technologyName Name of the scenario technology.
     */
    public static void setTechnology(String technologyName) {
        ThreadContext.put(ContextLevel.TECHNOLOGY.name(), technologyName);
    }

    /**
     * Put connection ID to context.
     * @param connectionIDValue Input name/connection ID.
     */
    public static void setConnectionID(String connectionIDValue) {
        ThreadContext.put(ContextLevel.CONNECTION_ID.name(), connectionIDValue);
    }

    /**
     * Put scenario phase to context.
     * @param scenarioTypeValue Phase of the scenario.
     */
    public static void setPhase(String scenarioTypeValue) {
        ThreadContext.put(ContextLevel.PHASE.name(), scenarioTypeValue);
    }

    /**
     * Put scenario name to context.
     * @param scenarioValue Name of the scenario.
     */
    public static void setScenario(String scenarioValue) {
        ThreadContext.put(ContextLevel.SCENARIO.name(), scenarioValue);
    }

    /**
     * Put workflow ID to context.
     * @param workflowIDValue workflow ID.
     */
    public static void setWorkflowExecutionId(String workflowIDValue){
        ThreadContext.put(ContextLevel.WORKFLOW_ID.name(), workflowIDValue);
    }

    /**
     * Put execution ID to context.
     * @param executionIDValue execution ID.
     */
    public static void setScenarioExecutionId(String executionIDValue){
        ThreadContext.put(ContextLevel.EXECUTION_ID.name(), executionIDValue);
    }

    /**
     * Remove technology from the context.
     */
    public static void unsetTechnology() {
        ThreadContext.remove(ContextLevel.TECHNOLOGY.name());
    }

    /**
     * Remove input name from the context.
     */
    public static void unsetConnectionID() {
        ThreadContext.remove(ContextLevel.CONNECTION_ID.name());
    }

    /**
     * Remove phase from the context.
     */
    public static void unsetPhase() {
        ThreadContext.remove(ContextLevel.PHASE.name());
    }

    /**
     * Remove scenaraio from the context.
     */
    public static void unsetScenario(){
        ThreadContext.remove(ContextLevel.SCENARIO.name());
    }

    /**
     * Set a context node.
     * @param contextNode Name of the particular context, generally name of a file.
     */
    public static void setContext(String contextNode){
        ThreadContext.put("CONTEXT", contextNode);
        ThreadContext.put("context-set", "true");
    }

    /**
     * Remove context node.
     */
    public static void clearContext(){
        ThreadContext.remove("CONTEXT");
        ThreadContext.remove("context-set");
    }
}
