package eu.profinit.manta.bachelor.thesis.logging.api.utils;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Formatter of messages - replaces placeholders with data.
 * @author jkovac
 */
public final class MessageFormatter {

    private static final String PLACEHOLDER_REGEX = "%\\{(.*?)\\}";
    private static final Pattern PATTERN = Pattern.compile(PLACEHOLDER_REGEX);

    /**
     * Private constructor.
     */
    private MessageFormatter() {
    }

    /**
     * Replaces names placeholders with values.
     * @param message Message with placeholders.
     * @param args Map with all placeholder regexes as keys and the placeholder values as keys.
     * @return Message with replaced placeholders.
     */
    public static String format(String message, Map<String, String> args) {
        String currentMessage = message;
        Matcher matcher = PATTERN.matcher(message);

        while (matcher.find()) {
            String placeholder = matcher.group(1);
            String placeholderRegex = "%\\{" + placeholder + "\\}";

            if (args.containsKey(placeholder)) {
                currentMessage = currentMessage.replaceAll(placeholderRegex, args.get(placeholder));
            }
        }

        return currentMessage;
    }

}
