package eu.profinit.manta.bachelor.thesis.logging.api.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;

/**
 * Parser of placeholders from error messages (user, technical and solution).
 * @author jkovac
 */
public final class MessageParser {

    private static final String PLACEHOLDER_REGEX = "%\\{(.*?)\\}";
    private static final Pattern PATTERN = Pattern.compile(PLACEHOLDER_REGEX);

    /**
     * Private constructor.
     */
    private MessageParser() {
    }

    /**
     * Parses placeholders from messages in order to create builder methods for them.
     *
     * Placeholders must be in the following format, otherwise they are ignored: %{placeholderName}
     *
     * @param error Annotation defining an error.
     *
     * @return List of all placeholders. Each placeholder is in the list once, even if it appears
     * multiple times across messages.
     */
    public static List<String> parsePlaceholders(Error error) {
        List<String> placeholders = new ArrayList<>();

        String userMessage = error.userMessage();
        parseMessage(placeholders, userMessage);

        String technicalMessage = error.technicalMessage();
        parseMessage(placeholders, technicalMessage);

        String solution = error.solution();
        parseMessage(placeholders, solution);

        return placeholders;
    }

    private static void parseMessage(List<String> placeholders, String message) {
        Matcher matcher = PATTERN.matcher(message);

        while (matcher.find()) {
            String placeholder = matcher.group(1);
            if (!placeholders.contains(placeholder)) {
                placeholders.add(placeholder);
            }
        }
    }

}
