package eu.profinit.manta.bachelor.thesis.logging.api.utils;

/**
 * Utility class used for converting error and category names from camel case to normalized format.
 * E.g. noConnectionMapping converts to NO_CONNECTION_MAPPING
 * @author jkovac
 */
public final class NameConverter {

    /**
     * Space between words in the converted name.
     */
    private static final String SPACE = "_";

    /**
     * Private constructor.
     */
    private NameConverter() {
    }

    /**
     * Converts camelcase name to format in which words are upper case and divided by underscore.
     * @param name String that should be converted.
     * @return The converted string.
     */
    public static String normalizeName(String name) {
        StringBuilder sb = new StringBuilder();

        final char[] nameArray = name.toCharArray();

        int currWordStartIndex = 0;
        char currChar = nameArray[currWordStartIndex];

        for (int i = currWordStartIndex + 1; i < nameArray.length; ++i) {
            char nextChar = nameArray[i];

            /*
                First char can be anything except upper case letter - number, special, letter, etc.
                The second one has to be an uppercase letter to append new word.
             */
            if (!Character.isUpperCase(currChar) && Character.isUpperCase(nextChar)) {
                appendWord(sb, nameArray, currWordStartIndex, i);
                currWordStartIndex = i;
            }

            currChar = nextChar;
        }

        // Append last word
        appendWord(sb, nameArray, currWordStartIndex, nameArray.length);

        return sb.toString();
    }

    private static void appendWord(StringBuilder sb, char[] wordArray, int startIndex, int endIndex) {
        if (sb.length() != 0) {
            sb.append(SPACE);
        }

        String newWord = new String(wordArray, startIndex, endIndex - startIndex);
        String newWordUpper = newWord.toUpperCase();
        sb.append(newWordUpper);
    }
}
