package ContextTests;

import java.io.File;

import org.junit.Assert;
import org.junit.Test;

import LoggingBase.Categories;
import Utils.TestUtils;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.LoggingContext;

public class ContextTests {

    private final Logger logger = new Logger(ContextTests.class);

    private static final String TECHNOLOGY = "TECHNOLOGY";
    private static final String CONNECTION = "CONNECTION";
    private static final String SCENARIO = "SCENARIO";
    private static final String PHASE = "PHASE";
    private static final String APPLICATION = "CLI";

    private final String LOG_FILE_PATH = "./target/test-classes/app.log";
    private final File logFile = new File(LOG_FILE_PATH);

    @Test
    public void testContext(){
        LoggingContext.setTechnology(TECHNOLOGY);
        LoggingContext.setScenario(SCENARIO);
        LoggingContext.setConnectionID(CONNECTION);
        LoggingContext.setPhase(PHASE);

        String fileName = "someFile.sql";

        LoggingContext.setContext(fileName);

        logger.log(Categories.testCategory().testError());

        Assert.assertTrue(TestUtils.fileContains(logFile, "Context:" + TECHNOLOGY + " -> " + CONNECTION + " -> " + PHASE + " -> " + SCENARIO + " -> CLI -> " + fileName));

        LoggingContext.unsetConnectionID();
        LoggingContext.unsetPhase();
        LoggingContext.unsetTechnology();
        LoggingContext.unsetScenario();
        LoggingContext.clearContext();
    }

    @Test
    public void testOverrideContext(){
        LoggingContext.setTechnology(TECHNOLOGY);
        LoggingContext.setScenario(SCENARIO);
        LoggingContext.setConnectionID(CONNECTION);
        LoggingContext.setPhase(PHASE);

        String fileName = "someFile.sql";

        LoggingContext.setContext(fileName);

        logger.log(Categories.testCategory().testError(), true);

        Assert.assertFalse(TestUtils.fileContains(logFile, "Context:" + TECHNOLOGY + " -> " + CONNECTION + " -> " + PHASE + " -> " + SCENARIO + " -> CLI -> " + fileName));
        Assert.assertTrue(TestUtils.fileContains(logFile, "Context:" + TECHNOLOGY + " -> " + CONNECTION + " -> " + PHASE + " -> " + SCENARIO + " -> CLI"));

        LoggingContext.unsetConnectionID();
        LoggingContext.unsetPhase();
        LoggingContext.unsetTechnology();
        LoggingContext.unsetScenario();
        LoggingContext.clearContext();
    }

    @Test
    public void testCorrectness(){
        LoggingContext.setTechnology(TECHNOLOGY);
        LoggingContext.setScenario(SCENARIO);
        LoggingContext.setConnectionID(CONNECTION);
        LoggingContext.setPhase(PHASE);

        logger.log(Categories.testCategory().testError());

        Assert.assertTrue(TestUtils.fileContains(logFile, "Context:" + TECHNOLOGY + " -> " + CONNECTION + " -> " + PHASE + " -> " + SCENARIO + " -> CLI"));

        LoggingContext.unsetConnectionID();
        LoggingContext.unsetPhase();
        LoggingContext.unsetTechnology();
        LoggingContext.unsetScenario();
        LoggingContext.clearContext();
    }

}
