package GenerationTests;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.junit.Test;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.BaseCategory;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class GeneratorTest extends BaseCategory {

    private String catchingName = "catching";

    @Test
    public void testSimpleErrorBuilder(){
        SimpleErrorBuilder simpleErrorBuilder = simpleError();
        Method[] methods = simpleErrorBuilder.getClass().getDeclaredMethods();
        assertEquals(1, methods.length);
        assertTrue(methods[0].getName().equals(catchingName));
    }

    @Test
    public void testOneParamBuilder(){
        OneParamBuilder oneParamBuilder = oneParam();
        Method[] methods = oneParamBuilder.getClass().getDeclaredMethods();
        assertEquals(2, methods.length);
        List<String> methodNames = Arrays.stream(methods).map(Method::getName).collect(Collectors.toList());

        assertTrue(methodNames.contains(catchingName));
        assertTrue(methodNames.contains("param"));
    }

    @Test
    public void testOneParamTwoMessages(){
        OneParamInTwoMessagesBuilder oneParamInTwoMessagesBuilder = oneParamInTwoMessages();
        Method[] methods = oneParamInTwoMessagesBuilder.getClass().getDeclaredMethods();
        assertEquals(2, methods.length);
        List<String> methodNames = Arrays.stream(methods).map(Method::getName).collect(Collectors.toList());

        assertTrue(methodNames.contains(catchingName));
        assertTrue(methodNames.contains("param"));
    }

    @Test
    public void testTwoParams(){
        TwoParamsBuilder twoParamsBuilder = twoParams();
        Method[] methods = twoParamsBuilder.getClass().getDeclaredMethods();
        assertEquals(3, methods.length);

        List<String> methodNames = Arrays.stream(methods).map(Method::getName).collect(Collectors.toList());

        assertTrue(methodNames.contains(catchingName));
        assertTrue(methodNames.contains("param1"));
        assertTrue(methodNames.contains("param2"));
    }

    @Test
    public void testThreeParams(){
        ThreeParamsBuilder threeParamsBuilder = threeParams();
        Method[] methods = threeParamsBuilder.getClass().getDeclaredMethods();
        assertEquals(4, methods.length);

        List<String> methodNames = Arrays.stream(methods).map(Method::getName).collect(Collectors.toList());

        assertTrue(methodNames.contains(catchingName));
        assertTrue(methodNames.contains("param1"));
        assertTrue(methodNames.contains("param2"));
        assertTrue(methodNames.contains("param3"));
    }

    @Test
    public void testIncorrectFormat_shouldNotHavetTheMethod(){
        IncorrectFormatBuilder incorrectFormatBuilder = incorrectFormat();
        Method[] methods = incorrectFormatBuilder.getClass().getDeclaredMethods();
        assertEquals(1, methods.length);

        List<String> methodNames = Arrays.stream(methods).map(Method::getName).collect(Collectors.toList());

        assertTrue(methodNames.contains(catchingName));
        assertFalse(methodNames.contains("param1"));
    }

    @Error(userMessage = "%{param1", technicalMessage = "", solution = "", severity = Severity.ERROR)
    public IncorrectFormatBuilder incorrectFormat(){
        return new IncorrectFormatBuilder(this);
    }

    @Error(userMessage = "%{param1}", technicalMessage = "%{param2}", solution = "%{param3}", severity = Severity.ERROR)
    public ThreeParamsBuilder threeParams(){
        return new ThreeParamsBuilder(this);
    }

    @Error(userMessage = "%{param1}", technicalMessage = "%{param2}", solution = "", severity = Severity.ERROR)
    public TwoParamsBuilder twoParams(){
        return new TwoParamsBuilder(this);
    }

    @Error(userMessage = "Test user message %{param}.", technicalMessage = "Test technical message %{param}.", solution = "", severity = Severity.ERROR)
    public OneParamInTwoMessagesBuilder oneParamInTwoMessages(){
        return new OneParamInTwoMessagesBuilder(this);
    }

    @Error(userMessage = "Test user message %{param}.", technicalMessage = "Test technical message.", solution = "Test solution.", severity = Severity.ERROR)
    public OneParamBuilder oneParam(){
        return new OneParamBuilder(this);
    }

    @Error(userMessage = "Test user message.", technicalMessage = "Test technical message.", solution = "Tech solution.", severity = Severity.ERROR)
    public SimpleErrorBuilder simpleError(){
        return new SimpleErrorBuilder(this);
    }

}
