package MessageTests;

import java.util.HashMap;
import java.util.Map;

import org.junit.Test;

import eu.profinit.manta.bachelor.thesis.logging.api.utils.MessageFormatter;

import static org.junit.Assert.assertEquals;

public class MessageFormatterTest {

    @Test
    public void formatTest_correctFormat(){
        String message = "Hello, my name is %{name} and I am %{age} old.";

        Map<String, String> args = new HashMap<>();
        args.put("name", "Josef Novak");
        args.put("age", Integer.toString(22));

        String formattedMessage = MessageFormatter.format(message, args);

        assertEquals("Hello, my name is Josef Novak and I am 22 old.", formattedMessage);
    }

    @Test
    public void formatTest_incorrectFormat(){
        String message = "Hello, my name is %{name} and I am %{age old.";

        Map<String, String> args = new HashMap<>();
        args.put("name", "Josef Novak");
        args.put("age", Integer.toString(22));

        String formattedMessage = MessageFormatter.format(message, args);

        assertEquals("Hello, my name is Josef Novak and I am %{age old.", formattedMessage);
    }

    @Test
    public void formatTest_correctFormat_replacesAll(){
        String message = "Hello, my name is %{name} and I am %{age} old. My brother is called %{name} as well.";

        Map<String, String> args = new HashMap<>();
        args.put("name", "Josef Novak");
        args.put("age", Integer.toString(22));

        String formattedMessage = MessageFormatter.format(message, args);

        assertEquals("Hello, my name is Josef Novak and I am 22 old. My brother is called Josef Novak as well.", formattedMessage);
    }
}
