package MessageTests;

import java.util.List;

import org.junit.Test;

import eu.profinit.manta.bachelor.thesis.logging.api.utils.MessageParser;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;

import static org.junit.Assert.*;

public class MessageParserTest {

    /**
     * ----------------------------------------------------- TESTS -----------------------------------------------------
     */

    @Test
    public void parsingTest_correctFormat_onePlaceholder() throws NoSuchMethodException {
        Error error = getClass().getMethod("onePlaceholder_correctFormat").getAnnotation(Error.class);
        List<String> placeHolders = MessageParser.parsePlaceholders(error);

        assertEquals(1, placeHolders.size());
        assertEquals(placeHolders.get(0), "name");
    }

    @Test
    public void parsingTest_incorrectFormat_onePlaceholder() throws NoSuchMethodException {
        Error error = getClass().getMethod("onePlaceholder_incorrectFormat").getAnnotation(Error.class);
        List<String> placeHolders = MessageParser.parsePlaceholders(error);

        assertEquals(0, placeHolders.size());
    }

    @Test
    public void parsingTest_correctFormat_onePlaceholder_twoTimes() throws NoSuchMethodException {
        Error error = getClass().getMethod("onePlaceholder_twoTimes_correctFormat").getAnnotation(Error.class);
        List<String> placeHolders = MessageParser.parsePlaceholders(error);

        assertEquals(1, placeHolders.size());
        assertEquals(placeHolders.get(0), "name");
    }

    @Test
    public void parsingTest_correctAndIncorrectFormat_onePlaceholder_twoTimes() throws NoSuchMethodException {
        Error error = getClass().getMethod("onePlaceholder_twoTimes_correctFormatAndIncorrect").getAnnotation(Error.class);
        List<String> placeHolders = MessageParser.parsePlaceholders(error);

        assertEquals(1, placeHolders.size());
        assertEquals(placeHolders.get(0), "name");
    }

    @Test
    public void parsingTest_correctFormat_twoPlaceholders() throws NoSuchMethodException {
        Error error = getClass().getMethod("twoPlaceholders_correctFormat").getAnnotation(Error.class);
        List<String> placeHolders = MessageParser.parsePlaceholders(error);

        assertEquals(2, placeHolders.size());
        assertTrue(placeHolders.contains("name"));
        assertTrue(placeHolders.contains("age"));
    }

    @Test
    public void parsingTest_oneHasIncorrectFormat_twoPlaceholders() throws NoSuchMethodException {
        Error error = getClass().getMethod("twoPlaceholders_oneHasIncorrectFormat").getAnnotation(Error.class);
        List<String> placeHolders = MessageParser.parsePlaceholders(error);

        assertEquals(1, placeHolders.size());
        assertTrue(placeHolders.contains("name"));
        assertFalse(placeHolders.contains("age"));
    }

    @Test
    public void parsingTest_correctFormat_threePlaceholders() throws NoSuchMethodException {
        Error error = getClass().getMethod("threePlaceholders_correctFormat").getAnnotation(Error.class);
        List<String> placeHolders = MessageParser.parsePlaceholders(error);

        assertEquals(3, placeHolders.size());
        assertTrue(placeHolders.contains("name"));
        assertTrue(placeHolders.contains("age"));
        assertTrue(placeHolders.contains("food"));
    }

    /**
     * ----------------------------------------------------- HELPER METHODS -----------------------------------------------------
     */

    @Error(
            userMessage = "Hello world, my name is %{name}.",
            technicalMessage = "",
            solution = "",
            severity = Severity.ERROR
    )
    public OnePlaceholder_correctFormatBuilder onePlaceholder_correctFormat(){ return null; }

    @Error(
            userMessage = "Hello world, my name is %{name.",
            technicalMessage = "",
            solution = "",
            severity = Severity.ERROR
    )
    public OnePlaceholder_incorrectFormatBuilder onePlaceholder_incorrectFormat(){ return null; }

    @Error(
            userMessage = "Hello world, my name is %{name}.",
            technicalMessage = "My name is %{name} and I am 22 years old.",
            solution = "",
            severity = Severity.ERROR
    )
    public OnePlaceholder_twoTimes_correctFormatBuilder onePlaceholder_twoTimes_correctFormat(){ return null; }

    @Error(
            userMessage = "Hello world, my name is %{name}.",
            technicalMessage = "My name is %{name and I am 22 years old.",
            solution = "",
            severity = Severity.ERROR
    )
    public OnePlaceholder_twoTimes_correctFormatAndIncorrectBuilder onePlaceholder_twoTimes_correctFormatAndIncorrect(){ return null; }

    @Error(
            userMessage = "Hello world, my name is %{name}.",
            technicalMessage = "My name is %{name} and I am %{age} years old.",
            solution = "",
            severity = Severity.ERROR
    )
    public TwoPlaceholders_correctFormatBuilder twoPlaceholders_correctFormat(){ return null; }

    @Error(
            userMessage = "Hello world, my name is %{name}.",
            technicalMessage = "My name is %{name} and I am %{age years old.",
            solution = "",
            severity = Severity.ERROR
    )
    public TwoPlaceholders_oneHasIncorrectFormatBuilder twoPlaceholders_oneHasIncorrectFormat(){ return null; }

    @Error(
            userMessage = "Hello world, my name is %{name}.",
            technicalMessage = "My name is %{name} and I am %{age} years old.",
            solution = "Let's go eat some %{food}.",
            severity = Severity.ERROR
    )
    public ThreePlaceholders_correctFormatBuilder threePlaceholders_correctFormat(){ return null; }
}
