package eu.profinit.manta.bachelor.thesis.logging.proof.of.concept.Errors.categories;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.BaseCategory;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;

/**
 * Errors holding errors regarding user authorization.
 * @author jkovac
 */
public class AuthorizationErrors extends BaseCategory {

    /**
     * Logged when a user inputs a wrong password.
     * @return bad password error builder
     */
    @Error(
            userMessage = "The user %{userName} has inputted an incorrect password.",
            technicalMessage = "The authorization of the user %{userName} failed due to an incorrect password.",
            solution = "Enter the correct password.",
            severity = Severity.FATAL
    )
    public BadPasswordBuilder badPassword(){
        return new BadPasswordBuilder(this);
    }

    /**
     * Logged when the user has an invalid license.
     * @return invalid license builder
     */
    @Error(
            userMessage = "The user %{userName} has an invalid license and is therefore unable to log in.",
            technicalMessage = "The user %{userName} has an invalid license. The hash of the user license: %{licenseHash}.",
            solution = "Upgrade the user license if the user has a right to have it.",
            severity = Severity.FATAL
    )
    public InvalidLicenseBuilder invalidLicense(){
        return new InvalidLicenseBuilder(this);
    }

    /**
     * Logged when a user tries to perform an action without having the necessary rights.
     * @return Missing user privilege builder
     */
    @Error(
            userMessage = "The user %{userName} has attempted to %{action} but lacks user privileges to do so.",
            technicalMessage = "The user %{userName} has attempted to %{action} but that user has a role %{userRole} which does not allow to perform this action." +
                    " The user must be a %{requiredRole} to be able to do this.",
            solution = "Change role settings for %{userName} or don't if this is the desired outcome.",
            severity = Severity.ERROR
    )
    public MissingUserPrivilegeBuilder missingUserPrivilege(){
        return new MissingUserPrivilegeBuilder(this);
    }

}
