package eu.profinit.manta.bachelor.thesis.logging.proof.of.concept.Errors.categories;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.BaseCategory;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;

/**
 * Errors occurring during input/output operations.
 * @author jkovac
 */
public class IOErrors extends BaseCategory {

    /**
     * Error logged when a given file is not found.
     * @return file not found builder
     */
    @Error(
            userMessage = "The wanted file was not found.",
            technicalMessage = "The wanted resource file was not found and therefore %{operation} will not run.",
            solution = "Check whether the file exists and check the correctness of the inputted path. Input path: %{path}.",
            severity = Severity.FATAL
    )
    public FileNotFoundBuilder fileNotFound(){
        return new FileNotFoundBuilder(this);
    }

    /**
     * Error logged when another process has locked the file and is therefore inaccessible
     * @return file locked builder
     */
    @Error(
            userMessage = "This file is locked.",
            technicalMessage = "This file is locked because another process is currently using it.",
            solution = "End the process that currently uses the context and try again.",
            severity = Severity.FATAL
    )
    public FileLockedBuilder fileLocked(){
        return new FileLockedBuilder(this);
    }

    /**
     * Error logged when a user inputted encoding is invalid
     * @return invalid encoding builder
     */
    @Error(
            userMessage = "Inputted encoding - %{encoding} - is not valid. The default encoding UTF-8 will be used instead.",
            technicalMessage = "Inputted encoding %{encoding} either does not exist or is not supported.",
            solution = "Check the table of supported encodings at %{website}. If the problem persists, contact the customer support at %{website}.",
            severity = Severity.WARN
    )
    public InvalidEncodingBuilder invalidEncoding(){
        return new InvalidEncodingBuilder(this);
    }

    /**
     * Error logged when a given file has an invalid format
     * @return invalid file format builder
     */
    @Error(
            userMessage = "The file format %{format} is not valid.",
            technicalMessage = "The file extension %{format} is not supported and was not loaded.",
            solution = "Please check that the format is supported. Allowed formats: %{allowedFormats}.",
            severity = Severity.ERROR
    )
    public InvalidFileFormatBuilder invalidFileFormat(){
        return new InvalidFileFormatBuilder(this);
    }

}
