package eu.profinit.manta.bachelor.thesis.logging.proof.of.concept.Errors.categories;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.BaseCategory;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;

/**
 * Category holding errors that stem from issues in the JVM.
 * @author jkovac
 */
public class SystemErrors extends BaseCategory {

    /**
     * Error logged when the JVM runs out of available memory for the garbage collector.
     * @return out of memory error builder
     */
    @Error(
            userMessage = "The JVM ran out of memory.",
            technicalMessage = "No more memory can be made available to the garbage collector.",
            solution = "Increase heap size and re-run the program.",
            severity = Severity.FATAL
    )
    public OutOfMemoryErrorBuilder outOfMemoryError(){
        return new OutOfMemoryErrorBuilder(this);
    }

    /**
     * Error logged due to stack overflow
     * @return stack overflow error builder
     */
    @Error(
            userMessage = "The recursive operation %{operation} has run too deep and caused a stack overflow.",
            technicalMessage = "Operation %{operation} has tried to use more memory than is available in the call stack.",
            solution = "Increase the stack bound and re-run the program.",
            severity = Severity.FATAL
    )
    public StackOverflowErrorBuilder stackOverflowError(){
        return new StackOverflowErrorBuilder(this);
    }



}
